<?php
/**
 * @version    5.1.0
 * @package    Com_Gavoting
 * @author     Glenn Arkell <glenn@glennarkell.com.au>
 * @copyright  2020 Glenn Arkell
 * @license    GNU General Public License version 2 or later; see LICENSE.txt
 */

namespace GlennArkell\Component\Gavoting\Administrator\Helper;

// No direct access
defined('_JEXEC') or die;

use \Joomla\CMS\Factory;
use \Joomla\CMS\Language\Text;
use \Joomla\CMS\Component\ComponentHelper;
use \Joomla\CMS\Response\JsonResponse;
use \Joomla\CMS\Table\Table;
use \Joomla\CMS\Application\SiteApplication;
use \GlennArkell\Component\Gavoting\Administrator\Helper\GavotingHelper;

/**
 * Gaajax helper.
 * @since  5.1.0
 */
class GaajaxHelper
{

	/**
	 * Create a selection list for preferential voting
	 * @param   object  $nom  nomination object
	 * @param   integer $qtyNoms number of nominations for this position
	 * @param   integer $posID  id reference of the position
	 * @return  selection list box
	 */
    public static function displayPrefVoteOptions($nom, $qtyNoms, $posID)
	{
        // Load admin language file
        $lang = Factory::getApplication()->getLanguage();
        $lang->load('com_gavoting', JPATH_ADMINISTRATOR);

        // setup array of the votes per option
        $nomVotes = \range(1,$qtyNoms,1);
        \arsort($nomVotes);
        $i=1;

        $selectList = '<div class="selectlist">';
		$selectList .= '<label id="jform_nom_ids_lbl" for="jform_nom_ids_'.$posID.'" class="selectlist">';
		$selectList .= '<div style="width:230px;float:left;">'.$nom->nomination_name.'</div>';
        $selectList .= '<div style="with:100px;float:right;">';
        $selectList .= '<select id="jform_nom_ids_'.$posID.'" name="jform[nom_ids][nom_id_'.$posID.'_'.$nom->id.']">';

        foreach ($nomVotes as $vote) {
            $nf = new \NumberFormatter('en_GB', \NumberFormatter::ORDINAL);
            $sel = $nf->format($i);

            if ($i == 1) {
                $selectList .= '<option value="0" >'.Text::_('GAVOTING_NO_VOTE').'</option>';
            }
            $selectList .= '<option value="'.$vote.'" >'.Text::sprintf('COM_GAVOTING_PREF_SELECTION', $sel).'</option>';
            $i++;
        }
        $selectList .= '</select></div></label></div>';
        
        return  $selectList;
	}

	/**
	 * Create a selection list for preferential voting
	 * @param   object  $nom  nomination object
	 * @param   integer $qtyNoms number of nominations for this position
	 * @param   integer $posID  id reference of the position
	 * @return  selection list box
	 */
    public static function displayMultiPrefVoteOptions($nom, $qtyNoms, $posID)
	{
        // Load admin language file
        $lang = Factory::getApplication()->getLanguage();
        $lang->load('com_gavoting', JPATH_ADMINISTRATOR);

        // setup array of the votes per option
        $nomVotes = \range(1,$qtyNoms,1);
        \arsort($nomVotes);
        $i=1;

        $selectList = '<div class="created-form">';
		$selectList .= '<label id="jform_nom_ids_lbl" for="jform_nom_ids_'.$posID.'" class="selectlist">';
		$selectList .= '<div style="width:230px;float:left;">'.$nom->nomination_name.'</div>';
        $selectList .= '<div style="with:100px;float:right;">';
        $selectList .= '<select id="jform_nom_ids_'.$posID.'" name="jform[nom_ids][nom_id_'.$posID.'_'.$nom->id.']">';

        foreach ($nomVotes as $vote) {
            $nf = new \NumberFormatter('en_GB', \NumberFormatter::ORDINAL);
            $sel = $nf->format($i);

            if ($i == 1) {
                $selectList .= '<option value="0" >'.Text::_('GAVOTING_NO_VOTE').'</option>';
            }
            $selectList .= '<option value="'.$vote.'" >'.Text::sprintf('COM_GAVOTING_PREF_SELECTION', $sel).'</option>';
            $i++;
        }
        $selectList .= '</select></div></label></div>';
        
        return  $selectList;
	}
}

