<?php

/**
 * @version    5.1.0
 * @package    Com_Gavoting
 * @author     Glenn Arkell <glenn@glennarkell.com.au>
 * @copyright  2020 Glenn Arkell
 * @license    GNU General Public License version 2 or later; see LICENSE.txt
 */

namespace GlennArkell\Component\Gavoting\Administrator\Helper;

// No direct access
defined('_JEXEC') or die;
                                      
use \Joomla\CMS\Factory;
use \Joomla\Data\DataObject;
use \Joomla\CMS\Language\Text;
use \Joomla\CMS\MVC\Model\ListModel;
use \Joomla\CMS\MVC\Model\BaseDatabaseModel;
use \Joomla\CMS\Installer\Installer;
use \Joomla\Filesystem\Path;
use \Joomla\CMS\Toolbar\Toolbar;
use \Joomla\CMS\Toolbar\ToolbarHelper;
use \Joomla\CMS\Uri\Uri;
use \Joomla\CMS\Component\ComponentHelper;
use \Joomla\CMS\User\UserHelper;
use \Joomla\CMS\User\UserFactoryInterface;
use \Joomla\CMS\Categories\Categories;
use \Joomla\CMS\Table\Table;
use \Joomla\CMS\Date\Date;
use \Joomla\CMS\Access\Access;
use \GlennArkell\Component\Gavoting\Administrator\Helper\GanotificationHelper;

/**
 * Main helper.
 * @since  1.6
 */
class GavotingHelper
{
	/**
	 * Build the HTTP query array
	 * @param array of the http query if the http query already prepared and we want to add more
	 * @param string view or a task
	 * @param string control method to be used
	 * @param string reference indicator
	 * @param string/int reference string or id
	 * @return array
	 * @ hint - this is used with http_build_query($query_string, '', '&amp;') to build a clean url
	 */
	public static function getHTTPQuery($existQ = null, $viewTask = 'view', $contModel = 'nomination', $ref = 'id', $linkId = 0)
	{
        if ($contModel == 'article') {
            $compName = 'com_content';
        } else {
            $compName = 'com_gavoting';
        }
        if (!$existQ) {
			$query_string = array();
			$query_string['option'] = $compName;
			$query_string[$viewTask] = $contModel;
			if ($ref) {
				$query_string[$ref] = $linkId;
			}
		} else {
			$query_string = $existQ;
			$query_string[$ref] = $linkId;
		}

		return $query_string;
	}

    /**
     * Gets the user record for the specific id reference
     */
    public static function getSpecificUser($id = 0)
	{
		if ($id) {
			$container = Factory::getContainer();
			$userFactory = $container->get(UserFactoryInterface::class);
			$user = $userFactory->loadUserById($id);
		} else {
			$user = Factory::getApplication()->getIdentity();
		}

		return $user;
	}

    /**
     * Gets the edit permission for an user
     * @param   mixed  $item  The item
     * @return  bool
     */
    public static function canUserEdit($item)
    {
        $permission = false;
        $user       = self::getSpecificUser();

        if ($user->authorise('core.edit', 'com_gavoting') || $user->authorise('core.vote', 'com_gavoting') || $user->authorise('core.nominate', 'com_gavoting')) {
            $permission = true;
        } else {
            if (isset($item->created_by)) {
                if ($user->authorise('core.edit.own', 'com_gavoting') && $item->created_by == $user->id) {
                    $permission = true;
                }
            } else {
                $permission = true;
            }
        }

        return $permission;
    }

	/**
	 * Get an instance of the named table
	 * @param   string  $name Table name
	 * @return null|object
	 */
	public static function getForeignTable($name = 'Nomination')
	{
		return BaseDatabaseModel::getTable($name, 'Administrator', $config = array());
	}

	/**
	 * Get an instance of the named model
	 * @param   string  $name  Model name
	 * @return null|object
	 */
	public static function getSiteModel($name = 'Nominations')
	{
		$model = null;

		// If the file exists, let's get it
		if (file_exists(JPATH_SITE . '/components/com_gavoting/src/Model/' . $name . 'Model.php')) {
			$model = BaseDatabaseModel::getInstance($name, 'Site');
		}

		return $model;
	}

	/**
	 * Get the version of the component
	 * @return version element of manifest
	 */
	public static function getComponentVersion()
	{
		$componentXML = Installer::parseXMLInstallFile(Path::clean(JPATH_ADMINISTRATOR . '/components/com_gavoting/gavoting.xml'));

		return $componentXML['version'];
	}

    /**
     * Gets todays date based on global timezone settings
     */
    public static function getTodaysDate()
	{

		//$tz = Factory::getConfig()->get('offset');
		//$date = Factory::getDate('now', $tz);
		//$today = date_format($date,'Y-m-d H:i:s');
		$today = Factory::getDate()->toSQL();

		return $today;
	}

    /**
     * Gets a category name from id reference
     */
    public static function getCategoryName($id) {

        // get the club names from the database
        $db		= Factory::getContainer()->get('DatabaseDriver');
		$query	= $db->getQuery(true);
        $query->clear();
		$query->select(' title ');
		$query->from(' #__categories ');
		$query->where(' id = '.(int) $id );
		$db->setQuery((string)$query);

	    try {
	        return $db->loadResult();
	    } catch (\RuntimeException $e) {
	        Factory::getApplication()->enqueueMessage($e->getMessage(), 'danger');
	        return false;
	    }
    }

	/**
	 * Get the record
	 * @params integer Record id reference
	 * @return object list of records
	 */
	public static function getArticle($id = 0)
	{
        $db		= Factory::getContainer()->get('DatabaseDriver');
		$query	= $db->getQuery(true);
        $query->clear();
		$query->select(' a.* ');
		$query->from(' #__content AS a ');
		$query->where('a.id = '.(int) $id);
		$db->setQuery((string)$query);
	    try {
	        return $db->loadObject();
	    } catch (\RuntimeException $e) {
	        Factory::getApplication()->enqueueMessage($e->getMessage(), 'danger');
	        return false;
	    }
	}

    /**
     * Gets a custom field record for a user_id and field_id
     */
    public static function getCustomField($user_id = 0, $field_id = 0) {

        // get the club names from the database
        $db		= Factory::getContainer()->get('DatabaseDriver');
		$query	= $db->getQuery(true);
        $query->clear();
		$query->select(' value ');
		$query->from(' #__fields_values ');
		$query->where(' field_id = '.(int) $field_id );
		$query->where(' item_id = '.$db->quote($user_id) );
		$db->setQuery((string)$query);

	    try {
	        return $db->loadResult();
	    } catch (RuntimeException $e) {
	        Factory::getApplication()->enqueueMessage($e->getMessage(), 'danger');
	        return false;
	    }
    }

    /**
     * Gets a custom field record for a user_id and field_id
     */
    public static function getCustomFields($context = 0) {

        // get the club names from the database
        $db		= Factory::getContainer()->get('DatabaseDriver');
		$query	= $db->getQuery(true);
        $query->clear();
		$query->select(' * ');
		$query->from(' #__fields ');
		$query->where(' context = '.$db->quote($context) );
		$query->where(' state = '.(int) 1 );
		$db->setQuery((string)$query);

	    try {
	        return $db->loadObjectList();
	    } catch (RuntimeException $e) {
	        Factory::getApplication()->enqueueMessage($e->getMessage(), 'danger');
	        return false;
	    }
    }

    /**
     * Gets all items relating to a specific custom field_id and value
     */
    public static function getAllCustomFieldRecs($field_id = 0, $rec_value = 0) {

        // get the club names from the database
        $db		= Factory::getContainer()->get('DatabaseDriver');
		$query	= $db->getQuery(true);
        $query->clear();
		$query->select(' item_id ');
		$query->from(' #__fields_values ');
		$query->where(' field_id = '.(int) $field_id );
		$query->where(' value = '. $db->quote($rec_value) );
		$db->setQuery((string)$query);

	    try {
	        return $db->loadColumn();
	    } catch (RuntimeException $e) {
	        Factory::getApplication()->enqueueMessage($e->getMessage(), 'danger');
	        return false;
	    }
    }

	public static function getListOptions($table = '#__gavoting_positions', $label = 'Position', $field = 'name' )
	{
        $params = ComponentHelper::getParams('com_gavoting');
        $sel_label = strtoupper('COM_GAVOTING_SELECT_'.$label);
        $sel_label = Text::_($sel_label);
        $db		= Factory::getContainer()->get('DatabaseDriver');
		$query	= $db->getQuery(true);
        $query->clear();
		$query->select(' "0" as "value", " - '.$sel_label.' - " as "text" UNION SELECT a.id as value, a.'.$field.' as text ');
		$query->from( $db->quoteName($table) . ' AS a ' );
		$query->where(' a.state = 1' );
		if ($table == '#__content') {
            $query->where(' a.catid = '.(int) $params->get('jd_cat', 0) );
        }
		$query->order(' text ASC ' );
		$db->setQuery((string)$query);

	    try {
	        $options = $db->loadObjectList();
	    } catch (\RuntimeException $e) {
	        Factory::getApplication()->enqueueMessage($e->getMessage(),'danger');
	        return false;
	    }

		return $options;
	}

	public static function getMembersOptions($nType = 'nomtors')
	{
		$user = self::getSpecificUser();
		$isAdmin = in_array(8,$user->groups);
        $params = ComponentHelper::getParams('com_gavoting');
        $single_mship = $params->get('single_mship',0);
        $proffield = $params->get('profile_field');
        $excl_user_gp = $params->get('excl_user_gp', 0);
		// get restrictions
		$restrict_noms = $params->get('restrict_'.$nType,0);
		$noms_user_gp = $params->get($nType.'_user_gp',0);

		if ($restrict_noms) {
			if ($nType == 'proxy') {
				$members = array();
				if (is_array($user->groups)) {
					foreach ($user->groups AS $gp) {
						$mmbrs = Access::getUsersByGroup($gp);
						$members = array_merge($mmbrs, $members);
					}
				}
			} else {
				$members = Access::getUsersByGroup($noms_user_gp);
			}
			$mbrList = implode(",",$members);
		}

		$allow_super = $params->get('allow_super',0);
		$restrict_users = $params->get('restrict_users',0);
		if ($restrict_users) {
			$listGPs = implode(',', $user->groups);
		}

		// get bundles
		$bundle_region = $params->get('bundle_region',0);
		$custom_profile = $params->get('custom_profile',1);
		$custom_user_fld = $params->get('custom_user_fld',0);
		$custom_cb_fld = $params->get('custom_cb_fld',0);
		$region = 0;
		$cfMbrs = 0;
		if ($bundle_region && !$isAdmin) {
			// if using the Profile field else use the selected custom field
			if ($custom_profile == 1 ) {
				$profile = UserHelper::getProfile($user->id);
                $fldName = $params->get('profile_user_fld',0);
                $fldArray = explode('.', $fldName);
                $fldValue = $profile->profile[$fldArray[1]];
				if (isset($profile->profile[$fldValue]) && $profile->profile[$fldValue] > '') {
					$region = $profile->profile[$fldValue];
				}
			} elseif ($custom_profile == 0 ) {
				if ($custom_user_fld) {
					// get this users custom field setting
					$user_custField_val = self::getCustomField($user->id, $custom_user_fld);
					$cfitems = self::getAllCustomFieldRecs($custom_user_fld, $user_custField_val);
					$cfMbrs = implode(",", $cfitems);
				}
			} elseif ($custom_profile == 2 ) {
				if ($custom_cb_fld) {
					$user_cbField_val = self::getCBuildUserValue($user->id, $custom_cb_fld);
				}
			}
		}

        $sel_member = Text::_('COM_GAVOTING_SELECT_MEMBER');
        $db		= Factory::getContainer()->get('DatabaseDriver');
		$query	= $db->getQuery(true);
        $query->clear();
		if ($single_mship) {
			$query->select(' "0" as "value", " - '.$sel_member.' - " as "text" UNION SELECT a.id as value, CONCAT(a.name," - ",IF(p.profile_value is null, "", REPLACE(p.profile_value,"\"",""))) as "text" ');
			$query->from( $db->quoteName('#__users') . ' AS a ' );
			$query->join('LEFT',' #__user_profiles AS p ON p.user_id = a.id AND p.profile_key = '.$db->quote($proffield));
		} else {
			$query->select(' "0" as "value", " - '.$sel_member.' - " as "text" UNION SELECT a.id as value, a.name as text ');
			$query->from( $db->quoteName('#__users') . ' AS a ' );
		}
		if ($restrict_users) {
			if ($allow_super && in_array(8,$user->groups)) {
				// no filter required
			} else {
				$query->join('LEFT', '#__user_usergroup_map AS ug ON ug.user_id = a.id ');
	            $query->where(' ug.group_id IN ('.$listGPs.')' );
			}
		}
		if ($bundle_region && $region) {
			$query->join("LEFT"," #__user_profiles AS r ON r.user_id = a.id AND r.profile_key = 'profile.region' AND REPLACE(r.profile_value,'\"','') = ".$db->quote($region));
			$query->where(' r.user_id IS NOT NULL ' );
		}
		if ($bundle_region && $cfMbrs) {
			$query->where(' a.id IN ('.$cfMbrs.')' );
		}
		// community builder field
        if ($bundle_region && $custom_cb_fld && !$isAdmin) {
			$query->join("LEFT"," #__comprofiler AS cb ON cb.user_id = a.id AND cb.".$db->quoteName($custom_cb_fld)." = ".$db->quote($user_cbField_val));
			$query->where(' cb.user_id IS NOT NULL ' );
		}
		$query->where(' a.block = 0' );
		if ($restrict_noms) {
			$query->where(' a.id IN ('.$mbrList.')' );
		}
		$query->order(' text ASC ' );
		$db->setQuery((string)$query);

	    try {
	        $options = $db->loadObjectList();
	    } catch (RuntimeException $e) {
	        Factory::getApplication()->enqueueMessage($e->getMessage(),'danger');
	        return false;
	    }

		if ($isAdmin) {
            $objList = $options;
        } else {
            // an array of objects is returned from the above
    	    $objList = array();
    	    $excl = false;
    		// cycle through list to exclude based on group
    	    foreach ($options AS $opt) {
    			if ($opt->value) {
    				// check if already voted - used for the proxy nomination process
                    $voted = self::hasVoted($opt->value, 0, 0);
                    if ($voted && $nType == 'proxy') { $excl = true; }
    
                    $u = self::getSpecificUser($opt->value);
    				if (is_array($excl_user_gp)) {
    					foreach ($excl_user_gp AS $gp) {
    						if (in_array($gp, $u->groups)) {
    	                        $excl = true;
    						}
    					}
    				}
    				if (!$excl) {
    					$objList[] = $opt;
    
    				} else {
    					$excl = false;
    				}
    			} else {
    				$objList[] = $opt;
    			}
    		}
		}

        Factory::getApplication()->setUserState('com_gavoting.test.data', $objList);
		return $objList;
	}

	public static function getProfileFieldsOptions()
	{
        $params = ComponentHelper::getParams('com_gavoting');
        $proflocal = $params->get('profile_suffix');
        $prof_key = 'profile'.$proflocal.'.';
        $keyLen = strlen($prof_key);
		$startkey = $keyLen+1;
		$sel_field = Text::_('COM_GAVOTING_SELECT_FIELD');

		// get the user records in the supplier group
        $db		= Factory::getContainer()->get('DatabaseDriver');
		$query	= $db->getQuery(true);
        $query->clear();
		$query->select(' "0" as "value", " - '.$sel_field.' - " as "text" UNION SELECT profile_key as value, substr(profile_key,'.$startkey.') as text ');
		$query->from( $db->quoteName('#__user_profiles') );
		$query->where(' substr(profile_key,1,'.$keyLen.') = '.$db->Quote($prof_key) );
		$query->group(' profile_key ' );
		$query->order(' text ASC ' );
		$db->setQuery((string)$query);

	    try {
	        $options = $db->loadObjectList();
	    } catch (\RuntimeException $e) {
	        Factory::getApplication()->enqueueMessage($e->getMessage(),'danger');
	        return false;
	    }

		return $options;
	}

	public static function getProfileOptions()
	{
		$sel_field = Text::_('COM_GAVOTING_SELECT_FIELD');
		// get the user profile records
        $db		= Factory::getContainer()->get('DatabaseDriver');
		$query	= $db->getQuery(true);
        $query->clear();
		$query->select(' "0" as "value", " - '.$sel_field.' - " as "text" UNION SELECT profile_key as value, SUBSTR(`profile_key`, LOCATE(".", `profile_key`)+1) as text ');
		$query->from( $db->quoteName('#__user_profiles') );
		$query->where(' substr(profile_key,1,11) != '.$db->quote('joomlatoken') );
		$query->group(' profile_key ' );
		$query->order(' text ASC ' );
		$db->setQuery((string)$query);

	    try {
	        $options = $db->loadObjectList();
	    } catch (RuntimeException $e) {
	        Factory::getApplication()->enqueueMessage($e->getMessage(),'danger');
	        return false;
	    }

		return $options;
	}

	public static function getProfileValue($id = 0, $field = null)
	{
        $db		= Factory::getContainer()->get('DatabaseDriver');
		$query	= $db->getQuery(true);
        $query->clear();
		$query->select(' a.* ');
		$query->from(' #__user_profiles a ');
        $query->where(' a.user_id = ' . (int) $id );
		$query->where(' a.profile_key = ' . $db->Quote($field) );
		$db->setQuery((string)$query);
	    try {
	        return $db->loadObject();
	    } catch (RuntimeException $e) {
	        Factory::getApplication()->enqueueMessage($e->getMessage(), 'danger');
	        return false;
	    }
	}

	public static function getCustomFieldValue($field = 0, $item = null)
	{
        $db		= Factory::getContainer()->get('DatabaseDriver');
		$query	= $db->getQuery(true);
        $query->clear();
		$query->select(' a.* ');
		$query->from(' #__fields_values a ');
        $query->where(' a.field_id = ' . (int) $field );
		$query->where(' a.item_id = ' . $db->Quote($item) );
		$db->setQuery((string)$query);
	    try {
	        return $db->loadObject();
	    } catch (RuntimeException $e) {
	        Factory::getApplication()->enqueueMessage($e->getMessage(), 'danger');
	        return false;
	    }
	}

	public static function getProfileRegionOptions()
	{
        $prof_key = 'profile.region';

		// get the user records in the supplier group
        $db		= Factory::getContainer()->get('DatabaseDriver');
		$query	= $db->getQuery(true);
        $query->clear();
		$query->select(' "0" as "value", " - Select Region - " as "text" UNION SELECT profile_value as value, profile_value as text ');
		$query->from( $db->quoteName('#__user_profiles') );
		$query->where(' profile_key = '.$db->Quote($prof_key) );
		$query->group(' profile_value ' );
		$query->order(' text ASC ' );
		$db->setQuery((string)$query);

	    try {
	        $options = $db->loadObjectList();
	    } catch (\RuntimeException $e) {
	        Factory::getApplication()->enqueueMessage($e->getMessage(),'danger');
	        return false;
	    }

		return $options;
	}

	public static function getCustomFieldOptions()
	{
		$sel_field = Text::_('COM_GAVOTING_SELECT_FIELD');
		// get the user custom field records
        $db		= Factory::getContainer()->get('DatabaseDriver');
		$query	= $db->getQuery(true);
        $query->clear();
		$query->select(' "0" as "value", " - '.$sel_field.' - " as "text" UNION SELECT id as value, title as text ');
		$query->from( $db->quoteName('#__fields') );
		$query->where(' context = "com_users.user"' );
		$query->order(' text ASC ' );
		$db->setQuery((string)$query);

	    try {
	        $options = $db->loadObjectList();
	    } catch (RuntimeException $e) {
	        Factory::getApplication()->enqueueMessage($e->getMessage(),'danger');
	        return false;
	    }

		return $options;
	}

	public static function getCBuildFieldOptions()
	{
		// check if community builder is installed
        $cbInstalled = ComponentHelper::isInstalled('com_comprofiler');

        $sel_field = Text::_('COM_GAVOTING_SELECT_FIELD');
		// get the user custom field records
        $db		= Factory::getContainer()->get('DatabaseDriver');
		$query	= $db->getQuery(true);
        $query->clear();
		if ($cbInstalled) {
            $query->select(' "0" as "value", " - '.$sel_field.' - " as "text" UNION SELECT name as value, title as text ');
    		$query->from( $db->quoteName('#__comprofiler_fields') );
    		$query->where(' published = '.(int) 1 );
		} else {
            $query->select(' "0" as "value", " - NOT INSTALLED - " as "text" ');
        }
		$query->order(' text ASC ' );
		$db->setQuery((string)$query);

	    try {
	        $options = $db->loadObjectList();
	    } catch (RuntimeException $e) {
	        Factory::getApplication()->enqueueMessage($e->getMessage(),'danger');
	        return false;
	    }

		return $options;
	}

	public static function getCBuildUserValue($id = 0, $field = null)
	{
        $db		= Factory::getContainer()->get('DatabaseDriver');
		$query	= $db->getQuery(true);
        $query->clear();
		$query->select($db->quotename($field));
		$query->from(' #__comprofiler ');
        $query->where(' user_id = ' . (int) $id );
		$db->setQuery((string)$query);
	    try {
	        return $db->loadResult();
	    } catch (RuntimeException $e) {
	        Factory::getApplication()->enqueueMessage($e->getMessage(), 'danger');
	        return false;
	    }
	}

	/**
	 * Get the nominations for the position record
	 * @params object position Record
	 * @return object list of records
	 * This is called from voterform.php to show a list of nominees for each position
	 */
	public static function getPositionNominations($pos = 0)
	{
        $params = ComponentHelper::getParams('com_gavoting');
		$user = self::getSpecificUser();
        $mbrList = implode(",",$user->groups);
        $req_sec_nom = $params->get('req_sec_nom', 1);
		// get bundles
		$bundle_region = $params->get('bundle_region',0);
		$custom_profile = $params->get('custom_profile',1); // 0 = custom, 1 = profile region, 2 = CB
		$custom_user_fld = $params->get('custom_user_fld',0);
		$custom_cb_fld = $params->get('custom_cb_fld',0);
		$region = false;
		$cfMbrs = false;
		$user_cbField_val = false;
		if ($bundle_region) {
			// if using the Profile field else use the selected custom field
			if ($custom_profile == 1 ) {
				$profile = UserHelper::getProfile($user->id);
				if (isset($profile->profile['region']) && $profile->profile['region'] > '') {
					$region = $profile->profile['region'];
				}
			} elseif ($custom_profile == 0 ) {
				if ($custom_user_fld) {
					// get this users custom field setting
					$user_custField_val = self::getCustomField($user->id, $custom_user_fld);
					$cfitems = self::getAllCustomFieldRecs($custom_user_fld, $user_custField_val);
					$cfMbrs = implode(",", $cfitems);
				}
			} elseif ($custom_profile == 2 ) {
				if ($custom_cb_fld) {
					$user_cbField_val = self::getCBuildUserValue($user->id, $custom_cb_fld);
				}
			}
		}

        $noms = array();

        $db		= Factory::getContainer()->get('DatabaseDriver');
		$query	= $db->getQuery(true);
        $query->clear();
		$query->select(' a.*, u.name as user_name, a.nom_name as nomination_name ');
		$query->select(' n.name as nominator_name, s.name as seconder_name');
		$query->select( $db->quote($pos->cat_id).' as cat_id, '.$db->quote($pos->pos_name).' as pos_name ');
		$query->select( $db->quote($pos->pos_qty).' as pos_qty, '.$db->quote($pos->position_id_name).' as position_id_name ');
		$query->from(' #__gavoting_nominations AS a ');
		$query->join(' LEFT',' #__users AS u ON u.id = a.nomination ');
		$query->join(' LEFT',' #__users AS n ON n.id = a.nom_id ');
		$query->join(' LEFT',' #__users AS s ON s.id = a.sec_id ');
		if ($pos->id) {
			$query->where(' a.position_id = '.(int) $pos->id);
		}
		$query->where(' a.state = 1 ');
		$query->where(' a.agreed = 1 ');
		if ($req_sec_nom) {
			$query->where(' a.sec_id > 0 ');
		}
		// filter only the nominations within bundle
        if ($bundle_region) {
            if ($region) {
    			$query->join("LEFT"," #__user_profiles AS r ON r.user_id = a.id AND r.profile_key = 'profile.region' AND REPLACE(r.profile_value,'\"','') = ".$db->quote($region));
    			$query->where(' r.user_id IS NOT NULL ' );
    		}
    		if ($cfMbrs) {
    			$query->where(' a.id IN ('.$cfMbrs.')' );
    		}
    		// community builder field
            if ($custom_cb_fld) {
    			$query->select(' cb.firstname, cb.cb_nsrcadistrict ');
                $query->join("LEFT"," #__comprofiler AS cb ON cb.user_id = a.nomination AND cb.".$db->quoteName($custom_cb_fld)." = ".$db->quote($user_cbField_val));
    			$query->where(' cb.user_id IS NOT NULL ' );
    		}
		}

		$query->order(' a.position_id ASC, a.id ASC ' );

		$db->setQuery((string)$query);
	    try {
	        $nominations = $db->loadObjectList();
	    } catch (RuntimeException $e) {
	        Factory::getApplication()->enqueueMessage($e->getMessage(), 'danger');
	        return false;
	    }

        if ($params->get('restrict_users',0)) {
            foreach ($nominations AS $nom) {
                $included = false;
                $n = self::getSpecificUser($nom->nomination);
                foreach ($n->groups AS $g) {
                    if (in_array($g, $user->groups) && !$included) {
                        $noms[] = $nom;
                        $included = true;
                    }
                }
            }
        } else {
            $noms = $nominations;
        }

        return $noms;
	}

	/**
	 * Get the nominations for the position record
	 * @params int position id
	 * @return object list of records
	 */
	public static function getNominationsByPosition($pos)
	{
        $params = ComponentHelper::getParams('com_gavoting');
		$user = self::getSpecificUser();
        $mbrList = implode(",",$user->groups);
        $req_sec_nom = $params->get('req_sec_nom', 1);
        $noms = array();

        $db		= Factory::getContainer()->get('DatabaseDriver');
		$query	= $db->getQuery(true);
        $query->clear();
		$query->select(' a.*, u.name as user_name, a.nom_name as nomination_name ');
		$query->select(' n.name as nominator_name, s.name as seconder_name');
		$query->select('p.cat_id, p.pos_name, p.pos_qty, p.pos_name as position_id_name ');
		$query->from(' #__gavoting_nominations AS a ');
		$query->join(' LEFT',' #__gavoting_positions AS p ON p.id = a.position_id ');
		$query->join(' LEFT',' #__users AS u ON u.id = a.nomination ');
		$query->join(' LEFT',' #__users AS n ON n.id = a.nom_id ');
		$query->join(' LEFT',' #__users AS s ON s.id = a.sec_id ');
		$query->where(' a.position_id = '.(int) $pos);
		$query->where(' a.state = 1 ');
		$query->where(' a.agreed = 1 ');
		if ($req_sec_nom) {
			$query->where(' a.sec_id > 0 ');
		}
		$query->order(' a.id ASC ' );

		$db->setQuery((string)$query);
	    try {
	        $nominations = $db->loadObjectList();
	    } catch (RuntimeException $e) {
	        Factory::getApplication()->enqueueMessage($e->getMessage(), 'danger');
	        return false;
	    }

        if ($params->get('restrict_users',0)) {
            foreach ($nominations AS $nom) {
                $included = false;
                $n = self::getSpecificUser($nom->nomination);
                foreach ($n->groups AS $g) {
                    if (in_array($g, $user->groups) && !$included) {
                        $noms[] = $nom;
                        $included = true;
                    }
                }
            }
        } else {
            $noms = $nominations;
        }

        return $noms;
	}

	/**
	 * Get the record
	 * @params integer Record id reference
	 * @return object list of records
	 */
	public static function getPosition($position = 0)
	{
		// get the broadcast types from the database
        $db		= Factory::getContainer()->get('DatabaseDriver');
		$query	= $db->getQuery(true);
        $query->clear();
		$query->select(' a.*, cat.title as election_year, CONCAT(cat.title," - ",a.pos_name) as position_name ');
		$query->from(' #__gavoting_positions AS a ');
		$query->join('LEFT',' #__categories AS cat ON cat.id = a.cat_id');
		$query->where('a.id = '.(int) $position);
		$db->setQuery((string)$query);
	    try {
	        return $db->loadObject();
	    } catch (\RuntimeException $e) {
	        Factory::getApplication()->enqueueMessage($e->getMessage(), 'danger');
	        return false;
	    }

	}

	/**
	 * Get the records
	 * @return object list of records
	 */
	public static function getPositions()
	{
        $db		= Factory::getContainer()->get('DatabaseDriver');
		$query	= $db->getQuery(true);
        $query->clear();
		$query->select(' a.*, CONCAT(cat.title," - ",a.pos_name) as position_id_name ');
		$query->from(' #__gavoting_positions AS a ');
		$query->join(' LEFT',' #__categories AS cat ON cat.id = a.cat_id ');
		$query->where(' a.state = 1 ');
		$db->setQuery((string)$query);
	    try {
	        return $db->loadObjectList();
	    } catch (\RuntimeException $e) {
	        Factory::getApplication()->enqueueMessage($e->getMessage(), 'danger');
	        return false;
	    }

	}

	/**
	 * Get the record
	 * @params array of data from form
	 * @return object user of the nominee with some profile data added
	 */
	public static function getNominationName($data = 0)
	{
        $params = ComponentHelper::getParams('com_gavoting');
		$ntd_user = self::getSpecificUser($data['nomination']);
        $ntd_user->profile = UserHelper::getProfile($ntd_user->id);
		if ($params->get('single_mship', 0)) {
			if ($data['oth_mbr']) {
				$localprof = 'profile'.$params->get('profile_suffix');
				$prof_key = str_replace($localprof.'.', '', $params->get('profile_field'));
				$ntd_user->nom_name = $ntd_user->profile->$localprof[$prof_key];
			} else {
				$ntd_user->nom_name = $ntd_user->name;
			}
		} else {
			$ntd_user->nom_name = $ntd_user->name;
		}

		return $ntd_user;
	}

	/**
	 * Get the record
	 * @params integer Record id reference
	 * @return object list of records
	 */
	public static function getNomination($nomination = 0)
	{
		// get the broadcast types from the database
        $db		= Factory::getContainer()->get('DatabaseDriver');
		$query	= $db->getQuery(true);
        $query->clear();
		$query->select(' a.*, u.name as user_name, a.nom_name as nomination_name, CONCAT(cat.title," - ",p.pos_name) as position_id_name ');
		$query->select(' n.name as nominator_name, s.name as seconder_name, p.cat_id, p.pos_name, p.pos_qty  ');
		$query->from(' #__gavoting_nominations AS a ');
		$query->join(' LEFT',' #__users AS u ON u.id = a.nomination ');
		$query->join(' LEFT',' #__users AS n ON n.id = a.nom_id ');
		$query->join(' LEFT',' #__users AS s ON s.id = a.sec_id ');
		$query->join(' LEFT',' #__gavoting_positions AS p ON p.id = a.position_id ');
		$query->join(' LEFT',' #__categories AS cat ON cat.id = p.cat_id ');
		if ($nomination) {
			$query->where(' a.id = '.(int) $nomination);
		}
		$query->where(' a.state = 1 ');
		$query->where(' a.agreed = 1 ');
		$query->order(' a.position_id ASC, a.nom_name ASC ' );

		$db->setQuery((string)$query);
	    try {
	        return $db->loadObjectList();
	    } catch (\RuntimeException $e) {
	        Factory::getApplication()->enqueueMessage($e->getMessage(), 'danger');
	        return false;
	    }

	}

	/**
	 * Check if nomination exists
	 * @params form data
	 * @return boolean
	 */
	public static function checkNomination($data)
	{

		$params = ComponentHelper::getParams('com_gavoting');
        $single_mship = $params->get('single_mship');
        $proffield = $params->get('profile_field');
        $proflocal = 'profile'.$params->get('profile_suffix');
        $u = self::getSpecificUser($data['nomination']);
        if ($single_mship) {
			if ($data['oth_mbr']) {
				$profile = UserHelper::getProfile($u->id);
				$prof_key = str_replace($proflocal.'.', '', $proffield);
				$nom_name = $profile->$proflocal[$prof_key];
			} else {
				$nom_name = $u->name;
			}
		} else {
			$nom_name = $u->name;
		}

		// count matching records to indicate nomination exists
        $db		= Factory::getContainer()->get('DatabaseDriver');
		$query	= $db->getQuery(true);
        $query->clear();
		$query->select(' count(id) ');
		$query->from(' #__gavoting_nominations ');
		$query->where(' state IN (0,1) ');
		$query->where(' nom_name = '.$db->Quote($nom_name));
		$query->where(' position_id = '. (int) $data['position_id']);

		$db->setQuery((string)$query);
	    try {
	        return $db->loadResult();
	    } catch (\RuntimeException $e) {
	        Factory::getApplication()->enqueueMessage($e->getMessage(), 'danger');
	        return false;
	    }

	}

	/**
	 * Get the record
	 * @params integer Record id reference
	 * @return object list of records
	 */
	public static function getTiedNominationPositions()
	{
		$nominations = array();
		$positions = array();
		$prevNom = new \stdClass();
		$prevNom->id = 0;
        // get the broadcast types from the database
        $db		= Factory::getContainer()->get('DatabaseDriver');
		$query	= $db->getQuery(true);
        $query->clear();
		$query->select(' a.position_id, p.pos_name, CONCAT(cat.title," - ",p.pos_name) as position_id_name, p.pos_qty ');
		$query->from(' #__gavoting_nominations AS a ');
		$query->join(' LEFT',' #__gavoting_positions AS p ON p.id = a.position_id ');
		$query->join(' LEFT',' #__categories AS cat ON cat.id = p.cat_id ');
		$query->where(' a.state = 1 ');
		$query->where(' a.agreed = 1 ');
		$query->group(' a.position_id' );
		$query->order(' a.position_id ASC' );

		$db->setQuery((string)$query);
	    try {
	        $pos = $db->loadObjectList();
	    } catch (\RuntimeException $e) {
	        Factory::getApplication()->enqueueMessage($e->getMessage(), 'danger');
	        return false;
	    }
	    
	    foreach ($pos AS $p) {
            // get all noms for this position
            $noms = self::checkPositionsVoteCount($p->position_id);

            // now check each list of nominations
            $cntr = 0;
            $prev_voteCnt = 0;
            if (is_array($noms) && (count($noms) > $p->pos_qty)) {
                foreach ($noms AS $nom) {

                    $cntr++;
                    if ($cntr <= $p->pos_qty) {
                        $prevNom = $nom;
                    } else {
                        if ($nom->votes == $prevNom->votes) {
                            if (!array_key_exists($prevNom->id,$nominations)) {
                                $nominations[$prevNom->id] = $prevNom;
                            }
                            $nominations[$nom->id] = $nom;
                        }
                    }
                }

                $positions[$p->position_id] = $nominations;
            }
	    }

        return array_filter($positions);
	}

	/**
	 * Get the record
	 * @params integer Record id reference
	 * @return object list of records
	 */
	public static function checkPositionsVoteCount($pos_id)
	{
		// get the broadcast types from the database
        $db		= Factory::getContainer()->get('DatabaseDriver');
		$query	= $db->getQuery(true);
        $query->clear();
		$query->select(' a.* ');
		$query->from(' #__gavoting_nominations AS a ');
		$query->where(' a.position_id = '. (int) $pos_id);
		$query->order(' a.votes DESC' );

		$db->setQuery((string)$query);
	    try {
	        return $db->loadObjectList();
	    } catch (\RuntimeException $e) {
	        Factory::getApplication()->enqueueMessage($e->getMessage().' - checking position vote count', 'danger');
	        return false;
	    }
	    


	}

	/**
	 * Check if user has voted
	 * @params integer user id reference
	 * @params integer motion indicator (yes-1,no-0)
	 * @params integer linkedUser if multi user
	 * @return boolean true on success
	 */
	public static function hasVoted($id = 0, $motion = 0, $linkedUser = 0)
	{
        $linkedUserRecs = self::getAllLinkedUsers($id, $linkedUser);

		$db		= Factory::getContainer()->get('DatabaseDriver');
		$query	= $db->getQuery(true);
        $query->clear();
		$query->select(' count(id) ');
		$query->from(' #__gavoting_voters ');
		$query->where(' state = 1 ');
		if ($linkedUser) {
			$query->where(' user_id IN ('. $linkedUserRecs .')' );
		} else {
			$query->where(' user_id = '. (int) $id);
	    }
		if ($motion) {
			$query->where(' motion_id = '. (int) $motion);
		} else {
			$query->where(' motion_id = 0 ');
	    }
		$db->setQuery((string)$query);
	    try {
	        return $db->loadResult();
	    } catch (\RuntimeException $e) {
	        Factory::getApplication()->enqueueMessage($e->getMessage(), 'danger');
	        return false;
	    }
	}

	/**
	 * Create a Vote record
	 * @params integer user id reference
	 * @params integer motion id reference
	 * @return boolean true on success
	 */
	public static function createVote($id = 0, $motion = 0)
	{
		$params = ComponentHelper::getParams('com_gavoting');
		$agm_date = new Date($params->get('agm_date'));
		$today = self::getTodaysDate();
		$electYr = self::getElectionYear();

		// Create and populate an object.
		$vote = new \stdClass();
		$vote->state     = 1;
		$vote->ordering  = 1;
		$vote->created_by   = $id;
		$vote->checked_out_time  = '0000-00-00 00:00:00';
		$vote->created_date  = $today;
		$vote->modified_by   = $id;
		$vote->modified_date  = $today;
		$vote->user_id   = $id;
		$vote->cat_id   = $electYr->id;
		$vote->motion_id = $motion;

		// Insert the object into the voters table.
	    try {
			$result = Factory::getContainer()->get('DatabaseDriver')->insertObject('#__gavoting_voters', $vote);
	    } catch (RuntimeException $e) {
	        Factory::getApplication()->enqueueMessage($e->getMessage(), 'danger');
	        $result = false;
	    }

        return $result;

	}

	/**
	 * Get Election Year
	 * @return integer category id
	 */
	public static function getElectionYear()
	{
        $extension = 'com_gavoting';
        $db		= Factory::getContainer()->get('DatabaseDriver');
		$query	= $db->getQuery(true);
        $query->clear();
		$query->select(' * ');
		$query->from(' #__categories ');
		$query->where(' extension = '.$db->Quote($extension) );
		$query->where(' published = 1 ' );
		$db->setQuery((string)$query);

	    try {
	        return $db->loadObject();
	    } catch (\RuntimeException $e) {
	        Factory::getApplication()->enqueueMessage($e->getMessage(), 'danger');
	        return false;
	    }
	}

	/**
	 * Get the record
	 * @params integer user id reference of the owner/principal of the vote
	 *       known as the voter_id and the user_id is actually 
	 *       the delegate casting the vote
	 * @return object of record
	 */
	public static function checkProxySet($id)
	{
		// get the record from the database
        $db		= Factory::getContainer()->get('DatabaseDriver');
		$query	= $db->getQuery(true);
        $query->clear();
		$query->select(' a.*, b.name, b.name AS proxy, c.name AS principal ');
		$query->from(' #__gavoting_proxys AS a');
		$query->join('LEFT', ' #__users AS b ON b.id = a.user_id');
		$query->join('LEFT', ' #__users AS c ON c.id = a.voter_id');
		$query->where(' a.state = 1 ');
		$query->where(' a.voter_id = '.(int)$id);
		$db->setQuery((string)$query);
	    try {
	        return $db->loadObject();
	    } catch (\RuntimeException $e) {
	        Factory::getApplication()->enqueueMessage($e->getMessage(), 'danger');
	        return false;
	    }

	}

	/**
	 * Get the record
	 * @params integer user id reference of an existing proxy setting
	 *       known as the user_id and the user_id is actually
	 *       the delegate casting the vote
	 * @return object of record
	 */
	public static function hasProxy($id)
	{
		// get the record from the database
        $db		= Factory::getContainer()->get('DatabaseDriver');
		$query	= $db->getQuery(true);
        $query->clear();
		$query->select(' a.*, b.name, b.name AS proxy, c.name AS principal ');
		$query->from(' #__gavoting_proxys AS a');
		$query->join('LEFT', ' #__users AS b ON b.id = a.user_id');
		$query->join('LEFT', ' #__users AS c ON c.id = a.voter_id');
		$query->where(' a.state = 1 ');
		$query->where(' a.user_id = '.(int)$id);
		$db->setQuery((string)$query);
	    try {
	        return $db->loadObject();
	    } catch (\RuntimeException $e) {
	        Factory::getApplication()->enqueueMessage($e->getMessage(), 'danger');
	        return false;
	    }

	}

	/**
	 * Get the record
	 * @params integer user id reference of the delegate of the vote
	 *       known as the user_id and the voter_id is actually
	 *       the owner/principal owning the vote
	 * @return object of record
	 */
	public static function countProxyVotes($id)
	{
		// get the record from the database
        $db		= Factory::getContainer()->get('DatabaseDriver');
		$query	= $db->getQuery(true);
        $query->clear();
		$query->select(' count(*) ');
		$query->from(' #__gavoting_proxys AS a');
		$query->join('LEFT', ' #__gavoting_voters AS b ON b.user_id = a.voter_id AND b.state = 1');
		$query->where(' a.state = 1 ');
		$query->where(' a.user_id = '.(int)$id);
		$query->where(' b.id IS NULL ');
		$db->setQuery((string)$query);
	    try {
	        return $db->loadResult();
	    } catch (\RuntimeException $e) {
	        Factory::getApplication()->enqueueMessage($e->getMessage(), 'danger');
	        return false;
	    }

	}

	/**
	 * Get the proxy records
	 * @params integer user id reference of the delegate of the vote
	 *       known as the user_id and the voter_id is actually
	 *       the owner/principal owning the vote
	 * @return object of record
	 */
	public static function getProxyVotes($id)
	{
		// get the record from the database
        $db		= Factory::getContainer()->get('DatabaseDriver');
		$query	= $db->getQuery(true);
        $query->clear();
		$query->select(' a.* ');
		$query->from(' #__gavoting_proxys AS a');
		$query->where(' a.state = 1 ');
		$query->where(' a.user_id = '.(int)$id);
		$db->setQuery((string)$query);
	    try {
	        return $db->loadObjectList();
	    } catch (\RuntimeException $e) {
	        Factory::getApplication()->enqueueMessage($e->getMessage(), 'danger');
	        return false;
	    }

	}

	/**
	 * Get the record
	 * @params integer user id reference of the delegate casting the vote
	 *       known as the user_id and the voter_id is actually
	 *       the owner of the vote
	 * @return object of record
	 */
	public static function getVoteownersOptions()
	{
		$id = self::getSpecificUser()->id;
		// get the record from the database
        $db		= Factory::getContainer()->get('DatabaseDriver');
		$query	= $db->getQuery(true);
        $query->clear();
		$query->select(' "0" as "value", " - Select Vote Owner - " as "text" UNION SELECT a.voter_id as value, b.name as text ');
		$query->from(' #__gavoting_proxys AS a');
		$query->join('LEFT', ' #__users AS b ON b.id = a.voter_id');
		$query->where(' a.state = 1 ');
		$query->where(' a.user_id = '.(int)$id);
		$db->setQuery((string)$query);
	    try {
	        return $db->loadObjectList();
	    } catch (\RuntimeException $e) {
	        Factory::getApplication()->enqueueMessage($e->getMessage(), 'danger');
	        return false;
	    }

	}

	/**
	 * Check date for closing nominations
	 * @return  boolean true on closed
	 */
    public static function nominationsClosed()
	{
		$params = ComponentHelper::getParams('com_gavoting');
		$close_noms = new Date($params->get('close_noms'));
		$today = new Date();
		$close_noms = $today > $close_noms ? true : false;

		return $close_noms;
	}

	/**
	 * Check date for closing voting
	 * @return  boolean true on closed
	 */
    public static function votingClosed()
	{
		$params = ComponentHelper::getParams('com_gavoting');
		$close_votes = new Date($params->get('close_votes'));
		$today = new Date();
		$votingClosed = $today > $close_votes ? true : false;

		return $votingClosed;
	}

	/**
	 * Check date for closing voting
	 * @return  boolean true on closed
	 */
    public static function votingOpen()
	{
		$params = ComponentHelper::getParams('com_gavoting');
		$open_votes = new Date($params->get('open_votes'));
		$today = new Date();
		$votingOpen = $today > $open_votes ? true : false;

		return $votingOpen;
	}

	/**
	 * Break up the name fields and form a display name version
	 * @param   object  $member  data including name, partner and the breakup firstnames and surnames.
	 * @return  string	combined name for the membership display
	 */
    public static function combineNames($member = null)
	{
		if (empty($member->partner) || $member->partner == '' || $member->partner == ' ') {
			$result = $member->name;
		} else {
			if (trim($member->surname) == trim($member->surnamep)) {
				$result = trim($member->firstname). ' & ' .trim($member->firstnamep) . ' ' . trim($member->surname);
			} else {
				$result = trim($member->name). ' & ' .trim($member->partner);
			}
		}

		return $result;

	}

	/**
	 * Get all the name fields as an object
	 * @param   id  $user id.
	 * @return  object	membership names object
	 */
    public static function breakdownNamesFromUserID($id = null)
	{
		$params = ComponentHelper::getParams('com_gavoting');
		$profsuf  = $params->get( 'profile_suffix' );
        $proffield = $params->get('profile_field');

        $db = Factory::getContainer()->get('DatabaseDriver');
        $query = $db->getQuery(true);
        $query->select(' a.id, a.name, a.email, a.block, a.registerDate, a.lastvisitDate ' );
        $query->select(' substr(`name`, 1, LOCATE(" ",`name`)) AS firstname ');
        $query->select(' if(substr(`name`, (LOCATE(" ",`name`)+1), 1)="&",substr(`name`, LOCATE(" ",`name`,(LOCATE(" ",`name`)+3))+1),substr(`name`, LOCATE(" ",`name`)+1)) AS surname ');
        $query->select(' if(h.profile_value IS NULL, "", h.profile_value) AS partner ');
        $query->select(' substr(h.profile_value, 1, LOCATE(" ",h.profile_value)) AS firstnamep ');
        $query->select(' substr(h.profile_value, LOCATE(" ",h.profile_value)+1) AS surnamep ');
        $query->from('`#__users` AS a');
        $query->join('LEFT', ' #__user_profiles AS h ON a.id = h.user_id AND h.profile_key = "profile'.$profsuf.'.'.$proffield.'" ');
        $query->where('a.id = ' . (int) $id );
        $db->setQuery($query);
		try {
			$member =  $db->loadObject();
		} catch (\RuntimeException $e) {
			Factory::getApplication()->enqueueMessage($e->getMessage(), 'danger');
			$member = false;
		}

		return $member;

	}

	/**
	 * Get the string of records linked to user
	 * @params integer Record id reference
	 * @return object list of records
	 */
	public static function getAllLinkedUsers($id = 0, $linkedId = 0)
	{
		$params = ComponentHelper::getParams('com_gavoting');
		$linkcust_field  = $params->get( 'linkcust_field' );

		// get the broadcast types from the database
        $db		= Factory::getContainer()->get('DatabaseDriver');
		$query	= $db->getQuery(true);
        $query->clear();
		$query->select(' a.* ');
		$query->from(' #__fields_values AS a ');
        $query->where(' a.field_id = ' . (int) $linkcust_field );
		$query->where(' ( a.item_id = '.$db->Quote($id) .' OR a.item_id = '.$db->Quote($linkedId) .' OR a.value = '.$db->Quote($id) .' OR a.value = '.$db->Quote($linkedId) .' ) ');
		$db->setQuery((string)$query);
	    try {
	        $records = $db->loadObjectList();
	    } catch (RuntimeException $e) {
	        Factory::getApplication()->enqueueMessage($e->getMessage(), 'danger');
	        return false;
	    }

	    if (!is_array($records)) {
			return 0;
		} else {
			$linkList = array();
			foreach ($records AS $rec) {
				// scroll through records to save the id reference only once
				if (!in_array($rec->item_id, $linkList)) {
					$linkList[] = $rec->item_id;
				}
				if (!in_array($rec->value, $linkList)) {
					$linkList[] = $rec->value;
				}
		    }
	
			$Listlink = '';
			foreach ($linkList AS $lnk) {
				// scroll through records to load the id reference as a string
				$lnk = is_numeric($lnk) ? $lnk : '"'.$lnk.'"';
				$Listlink .= $lnk . ',';
		    }
	        $Listlink = substr($Listlink,0,-1);
	
	        return $Listlink;
	    }
	}

	/**
	 * Check if member has access to perform action
	 * @params int User id reference
	 * @params int Group reference
	 * @return boolean
	 */
	public static function checkAccessGroup($user_id = 0, $group = 0)
	{
		$members = Access::getUsersByGroup($group);
		return in_array($user_id, $members) ? true : false;

	}

// 	/**
// 	 * Create a new record based on params
// 	 * @param   string  $table  Table name.
// 	 * @param   string  $key    The reference key
// 	 * @param   string  $value  Key value tyo be loaded
// 	 * @return boolean	false on failure
// 	 */
// 	public static function createNewRecord($table, $pos, $nom, $year, $value)
// 	{
//         //$state = $params->get('auto_create', 0) ? 1 : 0;
//         $new_rec = new \stdClass();
//         $new_rec->id = 0;
//         $new_rec->state = 1;
//         $new_rec->created_date = self::getTodaysDate();
//         $new_rec->created_by = self::getSpecificUser()->id;
//         $new_rec->position_id = $pos;
//         $new_rec->nomination = $nom;
//         $new_rec->cat_id = $year;
//         $new_rec->pref = $value;
//         $new_rec->comment = 'Auto created.';
// 
// 		try {
// 			$result = Factory::getContainer()->get('DatabaseDriver')->insertObject($table, $new_rec);
// 			Factory::getApplication()->enqueueMessage(Text::_('COM_GAVOTING_NEWREC_SUCCESSFUL'), 'notice');
// 		} catch (RuntimeException $e) {
// 			Factory::getApplication()->enqueueMessage($e->getMessage().' '.Text::_('COM_GAVOTING_NEWREC_FAILED'), 'danger');
// 			return false;
// 		}
// 		return true;
// 	}

// 	/**
// 	 * Setup preferential voting
// 	 * @params array $nom_ids submitted by voter
// 	 * Preferential Voting sample
//      *   [nom_ids] => Array
//      *       (
//      *           [nom_id_1_1] => 7_2     1 vote
//      *           [nom_id_1_2] => 13_1    2 votes
//      *           [nom_id_3_1] => 8_1     2 votes
//      *           [nom_id_3_2] => 14_2    1 vote
//      *           [nom_id_5_4] => 9_4     1 vote
//      *           [nom_id_5_1] => 10_1    4 votes
//      *           [nom_id_5_2] => 11_2    3 votes
//      *           [nom_id_5_3] => 12_3    2 votes
//      *       )
// 	 * @return array $nom_ids
// 	 */
// 	public static function setupPreferenceVoting($nom_ids, $elecYear)
// 	{
//         $updData = array();
//         foreach ($nom_ids as $keys => $values)
//         {
//             $k = explode('_',$keys);
//             $v = explode('_',$values);
//             self::createNewRecord('#__gavoting_preferences', $k[2], $v[0], $elecYear, $v[1]);
//         }
// 
//         foreach ($nom_ids as $keys => $values)
//         {
//             $k = explode('_',$keys);
//             $posID = $k[2];
//             $v = explode('_',$values);
//             $nomID = $v[0];
// 
//             // get all noms for a position in descending order of pref so our counter can make the votes
//             $noms = self::tallyPreferencePos($elecYear, $posID);
//             $cntr = \count($noms);
// 
//             for ($i=0; $i<=($cntr-1); $i++)
//             {
//                 if ($noms[$i]->pref) {
//                     if ($noms[$i]->nomination == $nomID) {
//                         $updData['nom_id_'.$posID.'_'.$nomID] = $i+1;
//                     }
//                 }
//             }
//         }
// 
//         return $updData;
// 	}

// 	/**
// 	 * Get the record
// 	 * @params integer election year (cat_id) reference
// 	 * @return object of record
// 	 */
// 	public static function tallyPreferencePos($elecYear, $posId)
// 	{
// 		// get the record from the database
//         $db		= Factory::getContainer()->get('DatabaseDriver');
// 		$query	= $db->getQuery(true);
//         $query->clear();
// 		$query->select(' nomination, pref ');
// 		$query->from(' #__gavoting_preferences');
// 		$query->where(' state = 1 ');
// 		$query->where(' cat_id = '.(int)$elecYear);
// 		$query->where(' position_id = '.(int)$posId);
// 		$query->order(' pref DESC');
// 		$db->setQuery((string)$query);
// 	    try {
// 	        return $db->loadObjectList();
// 	    } catch (\RuntimeException $e) {
// 	        Factory::getApplication()->enqueueMessage($e->getMessage(), 'danger');
// 	        return false;
// 	    }
// 
// 	}

	/**
	 * Update record
	 * @params integer nomination id reference
	 * @return boolean true on success
	 */
	public static function addVote($nomID = 0, $votes = 1)
	{
		$db		= Factory::getContainer()->get('DatabaseDriver');
		$db->setQuery((string)'SELECT * FROM #__gavoting_nominations WHERE id = '.$db->quote($nomID) );
		$nom = $db->loadObject();
		$nom->votes = $nom->votes + $votes;

		Factory::getContainer()->get('DatabaseDriver')->updateObject('#__gavoting_nominations', $nom, 'id');
		Factory::getApplication()->enqueueMessage(Text::sprintf('COM_GAVOTING_MEMBER_VOTE_FOR_SAVED',$nom->nom_name), 'message');

		return true;
	}

// 	/**
// 	 * Update record
// 	 * @params integer nomination id reference
// 	 * @return boolean true on success
// 	 */
// 	public static function cleanUpPreferences()
// 	{
// 		$userId = self::getSpecificUser()->id;
//         $db		= Factory::getContainer()->get('DatabaseDriver');
// 		$db->setQuery((string)'UPDATE #__gavoting_preferences SET created_by = 0, state = 2 WHERE created_by = '.$db->quote($userId) );
// 		$db->execute();
// 
// 		return true;
// 	}

}

