<?php

/**
 * @version    5.1.0
 * @package    Com_Gavoting
 * @author     Glenn Arkell <glenn@glennarkell.com.au>
 * @copyright  2020 Glenn Arkell
 * @license    GNU General Public License version 2 or later; see LICENSE.txt
 */

namespace GlennArkell\Component\Gavoting\Site\Controller;

// No direct access
defined('_JEXEC') or die;

use \Joomla\CMS\Factory;
use \Joomla\CMS\Router\Route;
use \Joomla\CMS\Language\Text;
use \Joomla\CMS\MVC\Controller\BaseController;
use \GlennArkell\Component\Gavoting\Administrator\Helper\GavotingHelper;

/**
 * Voter controller class.
 *
 * @since  1.6
 */
class MotionController extends BaseController
{
	/**
	 * Method to check out an item for editing and redirect to the edit form.
	 * @return void
	 * @since    1.6
     * @throws Exception
	 */
	public function edit()
	{
		$app = Factory::getApplication();

		// Get the previous edit id (if any) and the current edit id.
		$previousId = (int) $app->getUserState('com_gavoting.edit.motion.id');
		$editId     = $app->input->getInt('id', 0);

		// Set the user id for the user to edit in the session.
		$app->setUserState('com_gavoting.edit.motion.id', $editId);

		// Get the model.
		$model = $this->getModel('Motion', 'Site');

		// Check out the item
		if ($editId)
		{
			$model->checkout($editId);
		}

		// Check in the previous user.
		if ($previousId && $previousId !== $editId)
		{
			$model->checkin($previousId);
		}

		// Redirect to the edit screen.
		$this->setRedirect(Route::_('index.php?option=com_gavoting&view=motionform&layout=edit', false));
	}

	/**
	 * Method to save a user's profile data.
	 * @return    void
	 * @throws Exception
	 * @since    1.6
	 */
	public function publish()
	{
		// Initialise variables.
		$app = Factory::getApplication();

		// Checking if the user can remove object
		$user = GavotingHelper::getSpecificUser();

		if ($user->authorise('core.edit', 'com_gavoting') || $user->authorise('core.edit.state', 'com_gavoting'))
		{
			$model = $this->getModel('Motion', 'Site');

			// Get the user data.
			$id    = $app->input->getInt('id');
			$state = $app->input->getInt('state');

			// Attempt to save the data.
			$return = $model->publish($id, $state);

			// Check for errors.
			if ($return === false)
			{
				$this->setMessage(Text::sprintf('Save failed: %s', $model->getError()), 'warning');
			}

			// Clear the profile id from the session.
			$app->setUserState('com_gavoting.edit.motion.id', null);

			// Flush the data from the session.
			$app->setUserState('com_gavoting.edit.motion.data', null);

			// Redirect to the list screen.
			$this->setMessage(Text::_('COM_GAVOTING_ITEM_SAVED_SUCCESSFULLY'), 'success');
			$menu = Factory::getApplication()->getMenu();
			$item = $menu->getActive();

			if (!$item)
			{
				// If there isn't any menu item active, redirect to list view
				$this->setRedirect(Route::_('index.php?option=com_gavoting&view=motions', false));
			}
			else
			{
                $this->setRedirect(Route::_('index.php?Itemid='. $item->id, false));
			}
		}
		else
		{
			throw new \Exception(500);
		}
	}

	/**
	 * Remove data
	 * @return void
	 * @throws Exception
	 */
	public function remove()
	{
		// Initialise variables.
		$app = Factory::getApplication();

		// Checking if the user can remove object
		$user = GavotingHelper::getSpecificUser();

		if ($user->authorise('core.delete', 'com_gavoting'))
		{
			$model = $this->getModel('Motion', 'Site');

			// Get the user data.
			$id = $app->input->getInt('id', 0);

			// Attempt to save the data.
			$return = $model->delete($id);

			// Check for errors.
			if ($return === false)
			{
				$this->setMessage(Text::sprintf('Delete failed', $model->getError()), 'warning');
			}
			else
			{
				// Check in the profile.
				if ($return)
				{
					$model->checkin($return);
				}

                $app->setUserState('com_gavoting.edit.motion.id', null);
                $app->setUserState('com_gavoting.edit.motion.data', null);

                $app->enqueueMessage(Text::_('COM_GAVOTING_ITEM_DELETED_SUCCESSFULLY'), 'success');
                $app->redirect(Route::_('index.php?option=com_gavoting&view=motions', false));
			}

			// Redirect to the list screen.
			$menu = Factory::getApplication()->getMenu();
			$item = $menu->getActive();
			$this->setRedirect(Route::_($item->link, false));
		}
		else
		{
			throw new \Exception(500);
		}
	}

	/**
	 * Remove data
	 * @return void
	 * @throws Exception
	 */
	public function removeFile()
	{
		// Initialise variables.
		$app = Factory::getApplication();

		// Checking if the user can remove object
		$user = GavotingHelper::getSpecificUser();

		if ($user->authorise('core.delete', 'com_gavoting'))
		{
			$model = $this->getModel('Motion', 'Site');

			// Get the user data.
			$id = $app->input->getInt('id', 0);

			// Attempt to update the data.
			$return = $model->deleteFile($id);

			// Check for errors.
			if ($return === false) {
				$this->setMessage(Text::sprintf('Delete failed', $model->getError()), 'warning');
			} else {
                $app->setUserState('com_gavoting.edit.motion.id', null);
                $app->setUserState('com_gavoting.edit.motion.data', null);

                $app->enqueueMessage(Text::_('COM_GAVOTING_FILE_DELETED_SUCCESSFULLY'), 'success');
                $app->redirect(Route::_('index.php?option=com_gavoting&view=motions', false));
				// Check in the profile.
				$model->checkin($id);
			}

			// Redirect to the list screen.
			$menu = Factory::getApplication()->getMenu();
			$item = $menu->getActive();
			$url  = (empty($item->link) ? 'index.php?option=com_gavoting&view=motions' : $item->link);
			$this->setRedirect(Route::_($url, false));
		}
		else
		{
			throw new \Exception(500);
		}
	}

	/**
	 * Second the motion
	 * @return void
	 * @throws Exception
	 */
	public function seconder()
	{
		// Initialise variables.
		$app = Factory::getApplication();

		// Checking if the user can action object
		$user = GavotingHelper::getSpecificUser();

		if ($user->authorise('core.motion', 'com_gavoting')) {
			$id = $app->input->getInt('id', 0);
			$app->setUserState('com_gavoting.edit.motion.id', $id);

			$model = $this->getModel('Motion', 'Site');

			// Attempt to update the data.
			$return = $model->seconder($id, $user->id);

			// Check for errors.
			if ($return === false) {
				$this->setMessage(Text::sprintf('Seconder failed', $model->getError()), 'warning');
			} else {
				// Check in the profile.
				if ($return) {
					$model->checkin($return);
				}

                $app->setUserState('com_gavoting.edit.motion.id', null);

                $app->enqueueMessage(Text::_('COM_GAVOTING_AGREE_TO_SECOND_MOTION'), 'success');
                $app->redirect(Route::_('index.php?option=com_gavoting&view=motions', false));
			}
		}

	}

	/**
	 * Method to record a vote for motion
	 * @return void
	 */
	public function voteFor()
    {
        $app   = Factory::getApplication();
        $model = $this->getModel('Motion', 'Site');
        $pk    = $app->input->getInt('id');

        // Attempt to update the data
        try
        {
            $return = $model->voteFor($pk);

            // Clear the record id from the session.
            $app->setUserState('com_gavoting.edit.motion.id', null);

            // Redirect to the list screen
            $this->setMessage(Text::_('COM_GAVOTING_ITEM_SAVED_SUCCESSFULLY'), 'success');
            $this->setRedirect(Route::_('index.php?option=com_gavoting&view=motions', false));

            // Flush the data from the session.
            $app->setUserState('com_gavoting.edit.motion.data', null);
        }
        catch (Exception $e)
        {
            $errorType = ($e->getCode() == '404') ? Text::_('ERROR') : 'warning';
            $this->setMessage($e->getMessage(), $errorType);
            $this->setRedirect('index.php?option=com_gavoting&view=motions');
        }
    }

	/**
	 * Method to record a vote against motion
	 * @return void
	 */
	public function voteAgainst()
    {
        $app   = Factory::getApplication();
        $model = $this->getModel('Motion', 'Site');
        $pk    = $app->input->getInt('id');

        // Attempt to update the data
        try
        {
            $return = $model->voteAgainst($pk);

            // Clear the record id from the session.
            $app->setUserState('com_gavoting.edit.motion.id', null);

            // Redirect to the list screen
            $this->setMessage(Text::_('COM_GAVOTING_ITEM_SAVED_SUCCESSFULLY'), 'success');
            $this->setRedirect(Route::_('index.php?option=com_gavoting&view=motions', false));

            // Flush the data from the session.
            $app->setUserState('com_gavoting.edit.motion.data', null);
        }
        catch (Exception $e)
        {
            $errorType = ($e->getCode() == '404') ? Text::_('ERROR') : 'warning';
            $this->setMessage($e->getMessage(), $errorType);
            $this->setRedirect('index.php?option=com_gavoting&view=motions');
        }
    }

	/**
	 * Method to record a vote decision motion
	 * @return void
	 */
	public function voteDecision()
    {
        $app   = Factory::getApplication();
        $model = $this->getModel('Motion', 'Site');
        $pk    = $app->input->getInt('id');

        // Attempt to update the data
        try
        {
            $return = $model->voteDecision($pk);

            // Clear the record id from the session.
            $app->setUserState('com_gavoting.edit.motion.id', null);

            // Redirect to the list screen
            $this->setMessage(Text::_('COM_GAVOTING_DECISION_SUCCESSFULLY'), 'success');
            $this->setRedirect(Route::_('index.php?option=com_gavoting&view=motions', false));

            // Flush the data from the session.
            $app->setUserState('com_gavoting.edit.motion.data', null);
        }
        catch (Exception $e)
        {
            $errorType = ($e->getCode() == '404') ? Text::_('ERROR') : 'warning';
            $this->setMessage($e->getMessage(), $errorType);
            $this->setRedirect('index.php?option=com_gavoting&view=motions');
        }
    }
}
