<?php
/**
 * @version    5.1.0
 * @package    Com_Gavoting
 * @author     Glenn Arkell <glenn@glennarkell.com.au>
 * @copyright  2020 Glenn Arkell
 * @license    GNU General Public License version 2 or later; see LICENSE.txt
 */

namespace GlennArkell\Component\Gavoting\Site\Model;

// No direct access.
defined('_JEXEC') or die;

use \Joomla\CMS\Factory;
use \Joomla\Utilities\ArrayHelper;
use \Joomla\CMS\Language\Text;
use \Joomla\CMS\Table\Table;
use \Joomla\CMS\MVC\Model\FormModel;
use \Joomla\CMS\Component\ComponentHelper;
use \Joomla\Filesystem\File;
use \Joomla\Filesystem\Folder;
use \Joomla\Filesystem\Path;
use \GlennArkell\Component\Gavoting\Administrator\Helper\GavotingHelper;
use \GlennArkell\Component\Gavoting\Administrator\Helper\GanotificationHelper;

/**
 * Gavoting model.
 *
 * @since  1.6
 */
class ProxyvoterformModel extends FormModel
{
    private $item = null;

    /**
     * Method to auto-populate the model state.
     * Note. Calling getState in this method will result in recursion.
     * @return void
     * @since  1.6
     * @throws Exception
     */
    protected function populateState()
    {
        $app = Factory::getApplication('com_gavoting');

        // Load state from the request userState on edit or from the passed variable on default
        if (Factory::getApplication()->input->get('layout') == 'edit')
        {
                $id = Factory::getApplication()->getUserState('com_gavoting.edit.proxyvoter.id');
        }
        else
        {
                $id = Factory::getApplication()->input->get('id');
                Factory::getApplication()->setUserState('com_gavoting.edit.proxyvoter.id', $id);
        }

        $this->setState('proxyvoter.id', $id);

        // Load the parameters.
        $params       = $app->getParams();
        $params_array = $params->toArray();

        if (isset($params_array['item_id']))
        {
                $this->setState('proxyvoter.id', $params_array['item_id']);
        }

        $this->setState('params', $params);
    }

    /**
     * Method to get an ojbect.
     * @param   integer $id The id of the object to get.
     * @return Object|boolean Object on success, false on failure.
     * @throws Exception
     */
    public function getItem($id = null)
    {
        if ($this->item === null)
        {
            $this->item = false;

            if (empty($id)) {
                    $id = $this->getState('proxyvoter.id');
            }

			$user = GavotingHelper::getSpecificUser();
			if (!$id) {
				// check if already set proxyvoter
				$proxyvoter = GavotingHelper::checkProxySet($user->id);
				if ($proxyvoter) { $id = $proxyvoter->id; }
			}

            // Get a level row instance.
            $table = $this->getTable();

            if ($table !== false && $table->load($id))
            {

                $id   = $table->id;
                

                $canEdit = $user->authorise('core.edit', 'com_gavoting') || $user->authorise('core.create', 'com_gavoting');

                if (!$canEdit && $user->authorise('core.edit.own', 'com_gavoting'))
                {
                        $canEdit = $user->id == $table->voter_id;
                }

                if (!$canEdit)
                {
                        throw new \Exception(Text::_('JERROR_ALERTNOAUTHOR'), 403);
                }

                // Check published state.
                if ($published = $this->getState('filter.published'))
                {
                        if (isset($table->state) && $table->state != $published)
                        {
                                return $this->item;
                        }
                }

                // Convert the Table to a clean stdClass.
                $properties = $table->getProperties(1);
                $this->item = ArrayHelper::toObject($properties, 'stdClass');
                
            }
        }
        
        return $this->item;
    }

    /**
     * Method to get the table
     * @param   string $type   Name of the Table class
     * @param   string $prefix Optional prefix for the table class name
     * @param   array  $config Optional configuration array for Table object
     * @return  Table|boolean Table if found, boolean false on failure
     */
    public function getTable($type = 'Proxyvoter', $prefix = 'Administrator', $config = array())
    {
        return parent::getTable($type, $prefix, $config);
    }

    /**
     * Get an item by alias
     * @param   string $alias Alias string
     * @return int Element id
     */
    public function getItemIdByAlias($alias)
    {
        $table      = $this->getTable();
        $properties = $table->getProperties();

        if (!in_array('alias', $properties))
        {
                return null;
        }

        $table->load(array('alias' => $alias));

        return $table->id;

    }

    /**
     * Method to check in an item.
     * @param   integer $id The id of the row to check out.
     * @return  boolean True on success, false on failure.
     * @since    1.6
     */
    public function checkin($id = null)
    {
        // Get the id.
        $id = (!empty($id)) ? $id : (int) $this->getState('proxyvoter.id');
        
        if ($id)
        {
            // Initialise the table
            $table = $this->getTable();

            // Attempt to check the row in.
            if (method_exists($table, 'checkin'))
            {
                if (!$table->checkIn($id))
                {
                    return false;
                }
            }
        }

        return true;
        
    }

    /**
     * Method to check out an item for editing.
     * @param   integer $id The id of the row to check out.
     * @return  boolean True on success, false on failure.
     * @since    1.6
     */
    public function checkout($id = null)
    {
        // Get the user id.
        $id = (!empty($id)) ? $id : (int) $this->getState('proxyvoter.id');
        
        if ($id)
        {
            // Initialise the table
            $table = $this->getTable();

            // Get the current user object.
            $user = GavotingHelper::getSpecificUser();

            // Attempt to check the row out.
            if (method_exists($table, 'checkout'))
            {
                if (!$table->checkOut($user->id, $id))
                {
                    return false;
                }
            }
        }

        return true;
        
    }

    /**
     * Method to get the form.
     * The base form is loaded from XML
     * @param   array   $data     An optional array of data for the form to interogate.
     * @param   boolean $loadData True if the form is to load its own data (default case), false if not.
     * @return    JForm    A JForm object on success, false on failure
     * @since    1.6
     */
    public function getForm($data = array(), $loadData = true)
    {
 		// Get the form.
        $form = $this->loadForm('com_gavoting.proxyvoter', 'proxyvoterform', array(
                        'control'   => 'jform',
                        'load_data' => $loadData
                )
        );

        if (empty($form))
        {
                return false;
        }
        
        return $form;

    }

    /**
     * Method to get the data that should be injected in the form.
     * @return    mixed    The data for the form.
     * @since    1.6
     */
    protected function loadFormData()
    {
        $data = Factory::getApplication()->getUserState('com_gavoting.edit.proxyvoter.data', array());

        if (empty($data))
        {
            $data = $this->getItem();
            $cat_id = GavotingHelper::getElectionYear();
			$data->cat_id = $cat_id->id;
			$data->cat_id_name = $cat_id->title;
            $user = GavotingHelper::getSpecificUser();
			if (!$user->authorise('core.manage', 'com_gavoting')) {
				$data->voter_id_name = $user->name;
	            $data->voter_id = $user->id;
			}
        }

        return $data;
    }

    /**
     * Method to save the form data.
     * @param   array $data The form data
     * @return bool
     * @throws Exception
     * @since 1.6
     */
    public function save($data)
    {
        $id    = (!empty($data['id'])) ? $data['id'] : (int) $this->getState('proxyvoter.id');
        //$data['id'] = 0;
        $data['state'] = 1;
        $user  = GavotingHelper::getSpecificUser();
        $authorised = $user->authorise('core.vote', 'com_gavoting');

        if ($authorised !== true) {
            Factory::getApplication()->enqueueMessage(Text::_('COM_GAVOTING_NOT_AUTHORISED_VOTE'), 'danger');
            return false;
        }

        $params = ComponentHelper::getParams('com_gavoting');
        $proxyFwd = $params->get('fwd_proxy',0);

		// check if user is changing proxy back self
		if ($data['id'] > 0 && ($data['user_id'] == $data['voter_id'])) {
			$data['state'] = 2;
		}

        if (!$id) {
            $data['created_date']  = GavotingHelper::getTodaysDate();
        }

        if (!$proxyFwd) {
			$hasProxy = GavotingHelper::hasProxy($data['user_id']);
			// advise previous principle that their proxy has nominated a proxy
			if ($hasProxy && $proxyFwd) {
				if (!$params->get('tmpl_mail',0)) {
                    GanotificationHelper::sendProxyNotification($data, $params);
                } else {
                }
			}
		}

        $table = $this->getTable();
        if ($table->save($data) === true) {
			Factory::getApplication()->enqueueMessage(Text::sprintf('COM_GAVOTING_PROXYVOTER_REGISTERED', $data['voter_id_name']), 'message');
	        if (!$params->get('tmpl_mail',0)) {
                GanotificationHelper::sendProxyNotification($data, $params);
            } else {
                $data['voter_id_name'] = $user->name;
            }
	        return true;
        } else {
			Factory::getApplication()->enqueueMessage(Text::sprintf('COM_GAVOTING_PROXYVOTER_FAILED', $mu->name), 'Warning');
			return false;
        }

    }

    /**
     * Method to delete data
     * @param   int $pk Item primary key
     * @return  int  The id of the deleted item
     * @throws Exception
     * @since 1.6
     */
    public function delete($pk)
    {
        $user = GavotingHelper::getSpecificUser();

        if (empty($pk)) {
            $pk = (int) $this->getState('proxyvoter.id');
        }

        if ($pk == 0 || $this->getItem($pk) == null) {
            throw new \Exception(Text::_('COM_GAVOTING_ITEM_DOESNT_EXIST'), 404);
        }

        if ($user->authorise('core.delete', 'com_gavoting') !== true) {
            throw new \Exception(Text::_('JERROR_ALERTNOAUTHOR'), 403);
        }

        $table = $this->getTable();
        $table->load($pk);
        $table->state = -2;

        if ($table->store($pk) !== true) {
            throw new \Exception(Text::_('JERROR_FAILED'), 501);
        }

        return $pk;
        
    }

    /**
     * Method to archive data
     * @param   int $pk Item primary key
     * @return  int  The id of the deleted item
     * @throws Exception
     * @since 1.6
     */
    public function archive($pk)
    {
        $user = GavotingHelper::getSpecificUser();

        if (empty($pk)) {
            $pk = (int) $this->getState('proxyvoter.id');
        }

        if ($pk == 0 || $this->getItem($pk) == null) {
            throw new \Exception(Text::_('COM_GAVOTING_ITEM_DOESNT_EXIST'), 404);
        }

        if ($user->authorise('core.delete', 'com_gavoting') !== true) {
            throw new \Exception(Text::_('JERROR_ALERTNOAUTHOR'), 403);
        }

        $table = $this->getTable();
        $table->load($pk);
        $table->state = 2;

        if ($table->store($pk) !== true) {
            throw new \Exception(Text::_('JERROR_FAILED'), 501);
        }

        return $pk;
        
    }

    /**
     * Check if data can be saved
     * @return bool
     */
    public function getCanSave()
    {
        $table = $this->getTable();

        return $table !== false;
    }
    
}
