<?php
/**
 * @version    5.1.0
 * @package    Com_Gavoting
 * @author     Glenn Arkell <glenn@glennarkell.com.au>
 * @copyright  2020 Glenn Arkell
 * @license    GNU General Public License version 2 or later; see LICENSE.txt
 */
// No direct access
defined('_JEXEC') or die;

use \Joomla\CMS\HTML\HTMLHelper;
use \Joomla\CMS\Factory;
use \Joomla\CMS\Uri\Uri;
use \Joomla\CMS\Router\Route;
use \Joomla\CMS\Language\Text;
use \Joomla\CMS\Layout\LayoutHelper;
use \Joomla\CMS\User\UserHelper;
use \Joomla\CMS\Date\Date;
use \GlennArkell\Component\Gavoting\Administrator\Helper\GavotingHelper;

// load any assets required
$this->document->getWebAssetManager()
    ->usePreset('com_gavoting.gavotingpreset');

// Load admin language file
$lang = Factory::getApplication()->getLanguage();
$lang->load('com_gavoting', JPATH_ADMINISTRATOR);

$user       = GavotingHelper::getSpecificUser();
$userId     = $user->id;
$listOrder  = $this->state->get('list.ordering', 'a.mov_date');
$listDirn   = $this->state->get('list.direction', 'ASC');
$canCreate  = $user->authorise('core.create', 'com_gavoting');
$canEdit    = $user->authorise('core.edit', 'com_gavoting');
$canCheckin = $user->authorise('core.manage', 'com_gavoting');
$canChange  = $user->authorise('core.edit.state', 'com_gavoting');
$canDelete  = $user->authorise('core.delete', 'com_gavoting');
$canNominate  = $user->authorise('core.nominate', 'com_gavoting');
$canVote  = $user->authorise('core.vote', 'com_gavoting');
$canMotion  = $user->authorise('core.motion', 'com_gavoting');
$canChair  = $user->authorise('core.chair', 'com_gavoting');

$showVotes = $this->params->get('show_votes',0);
$notag_colour = $this->params->get('notag_colour','#a51f18');
$req_sec_mot = $this->params->get('req_sec_mot',1);

// get restrictions
$restrict_nomees = $this->params->get('restrict_nomees',0);
$nomees_user_gp = $this->params->get('nomees_user_gp',0);

$cols = 5;
$padSpace = '&nbsp; &nbsp; &nbsp; &nbsp; &nbsp; &nbsp;';

if ($this->params->get('multi_users', 0)) {
	$linked_field = $this->params->get('linkcust_field', 0);
	$linkedUser = GavotingHelper::getCustomField($user->id, $linked_field);
} else {
    $linkedUser = 0;
}
$votingClosed = GavotingHelper::votingClosed();
$votingOpen = GavotingHelper::votingOpen();

/*
echo '<pre>Test<br />';
print_r($this->items);
echo '</pre>';
print_r(Factory::getApplication()->getUserState('com_gavoting.test.files'));
print_r(Factory::getApplication()->getUserState('com_gavoting.test.data'));
*/
?>

<h2><?php echo Text::_('COM_GAVOTING_TITLE_MOTIONS'); ?></h2>

<form action="<?php echo htmlspecialchars(Uri::getInstance()->toString()); ?>" method="post"
      name="adminForm" id="adminForm" class="com-content-category__articles">

	<?php
        // display standard joomla filter files
        //echo LayoutHelper::render('joomla.searchtools.default', array('view' => $this));
        // display component view specific filter files
        echo LayoutHelper::render('default_filter', array('view' => $this), dirname(__FILE__));
    ?>

    <div class="table-responsive">
	<table class="table table-striped" id="motionList">
		<thead>
		<tr>
			<th width="12%" class='center'>
				<?php echo HTMLHelper::_('grid.sort',  'COM_GAVOTING_MOTIONS_MOV_DATE', 'a.mov_date', $listDirn, $listOrder); ?>
			</th>
			<th width="20%" class="left">
				<?php echo HTMLHelper::_('grid.sort',  'COM_GAVOTING_MOTIONS_MOV_ID', 'mov_id_name', $listDirn, $listOrder); ?>
			</th>
			<th width="20%" class="left">
				<?php echo HTMLHelper::_('grid.sort',  'COM_GAVOTING_MOTIONS_SEC_ID', 'sec_id_name', $listDirn, $listOrder); ?>
			</th>
			<?php if ($canCheckin || $showVotes): ?>
				<th width="8%" class="center">
					<?php echo HTMLHelper::_('grid.sort',  'COM_GAVOTING_MOTIONS_VOTES_FOR', 'a.votes_for', $listDirn, $listOrder); ?>
				</th>
				<th width="8%" class="center">
					<?php echo HTMLHelper::_('grid.sort',  'COM_GAVOTING_MOTIONS_VOTES_AGAINST', 'a.votes_against', $listDirn, $listOrder); ?>
				</th>
			<?php endif; ?>
			<th width="5%" class="center">
				<?php echo HTMLHelper::_('grid.sort',  'COM_GAVOTING_MOTIONS_AGREED', 'a.agreed', $listDirn, $listOrder); ?>
			</th>
			<th width="12%" class="center">
				<?php echo HTMLHelper::_('grid.sort',  'COM_GAVOTING_MOTIONS_AGREED_DATE', 'a.agreed_date', $listDirn, $listOrder); ?>
			</th>

			<?php if (($canCheckin || $canDelete || $canVote) && $votingOpen): ?>
				<th width="15%" class="center">
					<?php echo Text::_('COM_GAVOTING_ACTIONS'); ?>
				</th>
			<?php endif; ?>

		</tr>
		</thead>
		<tfoot>
		<tr>
			<td colspan="<?php echo isset($this->items[0]) ? count(get_object_vars($this->items[0])) : 10; ?>">
				<?php echo $this->pagination->getListFooter(); ?>
			</td>
		</tr>
		</tfoot>
		<tbody>
		<?php foreach ($this->items as $i => $item) : ?>
			<?php
				$canEdit = $user->authorise('core.edit', 'com_gavoting');
				if (!$canEdit && $user->authorise('core.edit.own', 'com_gavoting')) {
					$canEdit = GavotingHelper::getSpecificUser()->id == $item->motion;
				}
				
				if (!$item->agreed && $item->agreed != null) {
					$nomStyle = ' style="color:'.$notag_colour.';"'; 
				} else { 
					$nomStyle = ''; 
				}
				
				// check if already voted for motion
				if ($canVote) {
					$hasVoted = GavotingHelper::hasVoted($userId, $item->id, $linkedUser);
				}
				
				if ($item->agreed) { 
                    $agreeIcon = '<i class="icon-publish"></i>'; 
                } else { 
                    $agreeIcon = '<i class="icon-unpublish"></i>'; 
                }
				
        		// set up the decision link of a motion record
        		$decideLink = GavotingHelper::getHTTPQuery(null, 'task', 'motion.voteDecision', 'id', $item->id);
                $decideURL = 'index.php?'.http_build_query($decideLink, '', '&amp;');
        		$forLink = GavotingHelper::getHTTPQuery(null, 'task', 'motion.voteFor', 'id', $item->id);
                $forURL = 'index.php?'.http_build_query($forLink, '', '&amp;');
        		$againstLink = GavotingHelper::getHTTPQuery(null, 'task', 'motion.voteAgainst', 'id', $item->id);
                $againstURL = 'index.php?'.http_build_query($againstLink, '', '&amp;');
			?>


			<tr class="row<?php echo $i % 2; ?>">
				<td<?php echo $nomStyle; ?>>
					<?php if (isset($item->checked_out) && $item->checked_out) : ?>
						<?php echo HTMLHelper::_('jgrid.checkedout', $i, $item->uEditor, $item->checked_out_time, 'motions.', $canCheckin); ?>
					<?php endif; ?>
					<?php if ($canEdit): ?>
						<a href="<?php echo Route::_('index.php?option=com_gavoting&view=motion&id='.(int) $item->id); ?>">
							<?php echo HtmlHelper::date($item->mov_date, Text::_('COM_GAVOTING_DISPLAY_DATE')); ?>
						</a>
					<?php else : ?>
						<?php echo HtmlHelper::date($item->mov_date, Text::_('COM_GAVOTING_DISPLAY_DATE')); ?>
					<?php endif; ?>
				</td>
				<td class="left"<?php echo $nomStyle; ?>>
					<?php echo $item->mov_id_name; ?>
				</td>
				<td class="left"<?php echo $nomStyle; ?>>
					<?php if ($req_sec_mot): ?>
						<?php if ($item->sec_id): ?>
							<?php echo $item->sec_id_name; ?>
						<?php else : ?>
							<?php if ($userId && $item->mov_id != $userId): ?>
								<a href="<?php echo Route::_('index.php?option=com_gavoting&task=motion.seconder&id='.(int) $item->id.'&user_id='.(int) $userId, false, 2); ?>"
									class="btn btn-mini" type="button" title="<?php echo Text::_('COM_GAVOTING_AGREE_TO_SECOND_MOTION'); ?>">
									<i class="fas fa-thumbs-up" ></i>
								</a>
							<?php else : ?>
								<?php echo Text::_('COM_GAVOTING_SECONDER_REQ'); ?>
							<?php endif; ?>
						<?php endif; ?>
					<?php else : ?>
						<?php echo Text::_('COM_GAVOTING_NO_SECONDER_REQ'); ?>
					<?php endif; ?>
				</td>
				<?php if ($canCheckin || $showVotes): ?>
	                <?php $cols = 7; ?>
					<td class="center">
						<?php echo $item->votes_for; ?>
					</td>
					<td class="center">
						<?php echo $item->votes_against; ?>
					</td>
				<?php endif; ?>

				<td class="center"<?php echo $nomStyle; ?>>
					<?php if (!is_null($item->agreed)) { echo $agreeIcon; }?>
				</td>
				<td class="center"<?php echo $nomStyle; ?>>
					<?php if (!is_null($item->agreed)) { echo HtmlHelper::date($item->agreed_date, Text::_('COM_GAVOTING_DISPLAY_DATE')); } ?>
				</td>

				<?php if (($canCheckin || $canDelete || $canVote) && $votingOpen): ?>
	                <?php $cols = $cols == 7 ? 8 : 6; ?>
					<td class="center">
						<?php /* --------------------  not voted yet --------------  */ ?>
						<?php if (!$hasVoted): ?>
							<?php if (is_null($item->agreed)): ?>
								
                                <?php if (!$votingClosed): ?>
									<?php if (!$req_sec_mot): ?>
										<a href="<?php echo Route::_($forURL, false, 2); ?>" class="btn btn-mini" type="button" title="<?php echo Text::_('COM_GAVOTING_VOTE_FOR'); ?>"><i class="icon-publish" ></i></a>
										<a href="<?php echo Route::_($againstURL, false, 2); ?>" class="btn btn-mini" type="button" title="<?php echo Text::_('COM_GAVOTING_VOTE_AGAINST'); ?>"><i class="icon-unpublish" ></i></a>
									<?php elseif (!$item->sec_id) : ?>
    		                            <?php echo Text::_('COM_GAVOTING_SECONDER_REQ'); ?>
    		                        <?php else : ?>
										<a href="<?php echo Route::_($forURL, false, 2); ?>" class="btn btn-mini" type="button" title="<?php echo Text::_('COM_GAVOTING_VOTE_FOR'); ?>"><i class="icon-publish" ></i></a>
										<a href="<?php echo Route::_($againstURL, false, 2); ?>" class="btn btn-mini" type="button" title="<?php echo Text::_('COM_GAVOTING_VOTE_AGAINST'); ?>"><i class="icon-unpublish" ></i></a>
                                    <?php endif; ?>
								<?php else : ?>
		                            <?php echo Text::_('COM_GAVOTING_VOTING_CLOSED'); ?>
								<?php endif; ?>
							
        						<?php if ($canCheckin): ?>
            						<a href="<?php echo Route::_('index.php?option=com_gavoting&task=motionform.edit&id=' . $item->id, false, 2); ?>" class="btn btn-mini" type="button" title="<?php echo Text::_('COM_GAVOTING_EDIT_RECORD'); ?>"><i class="icon-edit" ></i></a>
        						<?php endif; ?>

                            <?php else : ?>
								<?php echo Text::_('COM_GAVOTING_MOTION_ALREADY_DECIDED'); ?>
							<?php endif; ?>
						<?php /* --------------------  has voted already --------------  */ ?>
						<?php else : ?>
                            <?php echo Text::_('COM_GAVOTING_MOTION_ALREADY_VOTED'); ?>
						<?php endif; ?>

						<?php if (is_null($item->agreed)): ?>
                            <?php if ($canChair && ($item->votes_for != $item->votes_against)): ?>
    							<a href="<?php echo Route::_($decideURL, false, 2); ?>" class="btn btn-mini" type="button" title="<?php echo Text::_('COM_GAVOTING_VOTE_DECISION'); ?>"><i class="icon-featured" ></i></a>
        					<?php elseif ($canChair && ($item->votes_for == $item->votes_against)) : ?>
    							<br /><?php echo Text::_('COM_GAVOTING_VOTING_EQUAL'); ?></br />
                                <a href="<?php echo Route::_($forURL, false, 2); ?>" class="btn btn-mini" type="button" title="<?php echo Text::_('COM_GAVOTING_VOTE_FOR'); ?>"><i class="icon-publish" ></i></a>
    							<a href="<?php echo Route::_($againstURL, false, 2); ?>" class="btn btn-mini" type="button" title="<?php echo Text::_('COM_GAVOTING_VOTE_AGAINST'); ?>"><i class="icon-unpublish" ></i></a>
    						<?php endif; ?>
						<?php endif; ?>
						<?php if ($canDelete): ?>
							<a href="<?php echo Route::_('index.php?option=com_gavoting&task=motionform.remove&id=' . $item->id, false, 2); ?>" class="btn btn-mini delete-button" type="button" title="<?php echo Text::_('COM_GAVOTING_DELETE_RECORD'); ?>"><i class="icon-trash" ></i></a>
							<a href="<?php echo Route::_('index.php?option=com_gavoting&task=motionform.archive&id=' . $item->id, false, 2); ?>" class="btn btn-mini" type="button" title="<?php echo Text::_('COM_GAVOTING_ARCHIVE_RECORD'); ?>"><i class="icon-archive" ></i></a>
						<?php endif; ?>
					</td>
				<?php endif; ?>

			</tr>
			<tr>
				<td<?php echo $nomStyle; ?> colspan="<?php echo $cols; ?>">
					<?php echo $item->motion; ?>
					<?php
                        if (!is_null($item->agreed)) {
                            if ($item->agreed) {
                                echo '<p> </p><p><strong>';
                                echo Text::sprintf('COM_GAVOTING_MOTION_CARRIED',$item->votes_for, $item->votes_against);
                                echo '<span style="float:right;">'.Text::_('COM_GAVOTING_CARRIED').'</span></strong></p>';
                            } else {
                                echo '<p> </p><p><strong>';
                                echo Text::sprintf('COM_GAVOTING_MOTION_FAILED',$item->votes_against, $item->votes_for);
                                echo '<span style="float:right;">'.Text::_('COM_GAVOTING_FAILED').'</span></strong></p>';
                            }
                        }
                    ?>
				</td>
			</tr>
		<?php endforeach; ?>
		</tbody>
	</table>
    </div>

    <div class="table-responsive">
	<?php if ($canCreate && $canMotion && !$votingClosed) : ?>
		<a href="<?php echo Route::_('index.php?option=com_gavoting&task=motionform.edit&id=0', false, 0); ?>"
		   class="btn btn-success"><i class="icon-plus"></i> <?php echo Text::_('COM_GAVOTING_ADD_ITEM'); ?>
		</a>
	<?php endif; ?>
	<a href="<?php echo Route::_('index.php?option=com_gavoting&view=motions&layout=history', false, 0); ?>"
		class="btn btn-info right"><i class="icon-search"></i> <?php echo Text::_('COM_GAVOTING_VIEW_HISTORY'); ?>
	</a>
    </div>

	<input type="hidden" name="task" value=""/>
	<input type="hidden" name="boxchecked" value="0"/>
	<input type="hidden" name="filter_order" value="<?php echo $listOrder; ?>"/>
	<input type="hidden" name="filter_order_Dir" value="<?php echo $listDirn; ?>"/>
	<?php echo HTMLHelper::_('form.token'); ?>
</form>

<div style="margin:10px auto;width:75%;">
    <p class="small center"><?php echo Text::_('COM_GAVOTING_MOTION_LIST_LEGEND'); ?></p>
    <p class="small" style="margin-left: 60px;">
    	<a href="#" class="btn btn-mini" type="button"><i class="icon-publish" ></i></a> <?php echo Text::_('COM_GAVOTING_VOTE_FOR').$padSpace; ?>
    	<a href="#" class="btn btn-mini" type="button"><i class="icon-unpublish" ></i></a> <?php echo Text::_('COM_GAVOTING_VOTE_AGAINST').$padSpace; ?>
    	<?php if ($canCheckin || $canDelete): ?>
    		<a href="#" class="btn btn-mini" type="button"><i class="icon-featured" ></i></a> <?php echo Text::_('COM_GAVOTING_VOTE_DECISION'); ?><br /><br />
    		<a href="#" class="btn btn-mini" type="button"><i class="icon-edit" ></i></a> <?php echo Text::_('COM_GAVOTING_EDIT_RECORD').$padSpace; ?>
    		<a href="#" class="btn btn-mini" type="button"><i class="icon-trash" ></i></a> <?php echo Text::_('COM_GAVOTING_DELETE_RECORD').$padSpace; ?>
    		<a href="#" class="btn btn-mini" type="button"><i class="icon-archive" ></i></a> <?php echo Text::_('COM_GAVOTING_ARCHIVE_RECORD'); ?>
    	<?php endif; ?>
    </p>
</div>

<?php if($canDelete) : ?>
<script type="text/javascript">

	jQuery(document).ready(function () {
		jQuery('.delete-button').click(deleteItem);
	});

	function deleteItem() {

		if (!confirm("<?php echo Text::_('COM_GAVOTING_DELETE_MESSAGE'); ?>")) {
			return false;
		}
	}
</script>
<?php endif; ?>
