<?php
/**
 * @version    5.1.0
 * @package    Com_Gavoting
 * @author     Glenn Arkell <glenn@glennarkell.com.au>
 * @copyright  2020 Glenn Arkell
 * @license    GNU General Public License version 2 or later; see LICENSE.txt
 */
// No direct access
defined('_JEXEC') or die;

use \Joomla\CMS\Factory;
use \Joomla\CMS\Language\Text;
use \Joomla\CMS\Router\Route;
use \Joomla\CMS\Uri\Uri;
use \Joomla\CMS\HTML\HTMLHelper;
use \GlennArkell\Component\Gavoting\Administrator\Helper\GavotingHelper;
use \GlennArkell\Component\Gavoting\Administrator\Helper\GamodalHelper;

// load any assets required
$this->document->getWebAssetManager()
    ->usePreset('com_gavoting.gavotingpreset');

// Load admin language file
$lang = Factory::getApplication()->getLanguage();
$lang->load('com_gavoting', JPATH_ADMINISTRATOR);

$user = GavotingHelper::getSpecificUser();

$canEdit = $user->authorise('core.edit', 'com_gavoting');
$canDelete = $user->authorise('core.delete', 'com_gavoting');
$canManage = $user->authorise('core.manage', 'com_gavoting');

if (!$canEdit && $user->authorise('core.edit.own', 'com_gavoting')) {
	$canEdit = $user->id == $this->item->nomination;
}

/*
echo '<pre>Test<br />';
print_r($this->item);
echo '</pre>';
print_r(Factory::getApplication()->getUserState('com_gavoting.test.data'));
*/
?>

<h2><?php echo Text::_('COM_GAVOTING_TITLE_NOMINATION'); ?></h2>

<div class="item_fields">

	<div class="row-fluid span12">
		<table class="table" style="margin-bottom: 30px;">
			<tbody>
			<tr><th><?php echo Text::_('COM_GAVOTING_FORM_LBL_NOMINATION_POSITION_ID'); ?></th>
				<td><?php echo $this->item->position_id_name; ?>
                	<?php if($canEdit): ?>
                		<?php
                            $jd = 0;
                            if ($this->item->jd_id) { 
                                $jd = GavotingHelper::getArticle($this->item->jd_id); 
                            }
                            if ($jd) {
                               $jdHtml = GamodalHelper::setupModalButton('view', 'article', 'id', $jd->id, 'modal', 'modalJD', 'info', '', 'COM_GAVOTING_FORM_LBL_POSITION_JD_ID', 'icon-info', $this->item->position_id_name);
                               echo $jdHtml;
                    		}
                        ?>
                	<?php endif; ?>
                </td>
			<tr>
			<tr><th><?php echo Text::_('COM_GAVOTING_FORM_LBL_NOMINATION_NOMINATION'); ?></th>
				<td><?php echo $this->item->nom_name; ?></td>
			<tr>
			<tr><th><?php echo Text::_('COM_GAVOTING_FORM_LBL_NOMINATION_NOM_DATE'); ?></th>
				<td><?php  echo !is_null($this->item->nom_date) ? HtmlHelper::date($this->item->nom_date, Text::_('COM_GAVOTING_DISPLAY_DATE')) : '' ;  ?></td>
			<tr>
			<tr><th><?php echo Text::_('COM_GAVOTING_FORM_LBL_NOMINATION_NOM_ID'); ?></th>
				<td><?php echo isset($this->item->nomby_name) ? $this->item->nomby_name : ''; ?></td>
			<tr>
			<tr><th><?php echo Text::_('COM_GAVOTING_FORM_LBL_NOMINATION_SEC_ID'); ?></th>
				<td><?php echo isset($this->item->secby_name) ? $this->item->secby_name : ''; ?></td>
			<tr>
			<tr><th><?php echo Text::_('COM_GAVOTING_FORM_LBL_NOMINATION_AGREED'); ?></th>
				<td><?php if ($this->item->agreed) { echo '<i class="icon-publish"></i>';} ?></td>
			<tr>
			<tr><th><?php echo Text::_('COM_GAVOTING_FORM_LBL_NOMINATION_AGREED_DATE'); ?></th>
				<td><?php  echo !is_null($this->item->agreed_date) ? HtmlHelper::date($this->item->agreed_date, Text::_('COM_GAVOTING_DISPLAY_DATE')) : '' ;  ?></td>
			<tr>
			<tr><th><?php echo Text::_('COM_GAVOTING_FORM_LBL_NOMINATION_VOTES'); ?></th>
				<td><?php echo $this->item->votes; ?></td>
			<tr>
			</tbody>
		</table>

		<a class="btn btn-secondary" href="<?php echo Route::_('index.php?option=com_gavoting&task=nominationform.cancel'); ?>" title="<?php echo Text::_('COM_GAVOTING_RETURN_DESC'); ?>">
		<i class="icon-undo"></i> <?php echo Text::_("COM_GAVOTING_RETURN"); ?>
		</a>

		<?php if($canEdit && $this->item->checked_out == 0): ?>

			<a class="btn btn-warning" href="<?php echo Route::_('index.php?option=com_gavoting&task=nomination.edit&id='.$this->item->id); ?>" title="<?php echo Text::_('COM_GAVOTING_EDIT_DESC'); ?>">
			<i class="icon-edit"></i> <?php echo Text::_("COM_GAVOTING_EDIT_ITEM"); ?>
			</a>

		<?php endif; ?>

		<?php if($canDelete) : ?>
			<a class="btn btn-danger delete-button pull-right" href="<?php echo Route::_('index.php?option=com_gavoting&task=nomination.remove&id='.$this->item->id); ?>">
				<i class="icon-edit"></i> <?php echo Text::_("COM_GAVOTING_DELETE_ITEM"); ?>
			</a>
			<script type="text/javascript">
				jQuery(document).ready(function () {
					jQuery('.delete-button').click(deleteItem);
				});
				function deleteItem() {
					if (!confirm("<?php echo Text::_('COM_GAVOTING_DELETE_MESSAGE'); ?>")) {
						return false;
					}
				}
			</script>
		<?php endif; ?>

	</div>

</div>

