<?php
/**
 * @version    5.1.0
 * @package    Com_Gavoting
 * @author     Glenn Arkell <glenn@glennarkell.com.au>
 * @copyright  2020 Glenn Arkell
 * @license    GNU General Public License version 2 or later; see LICENSE.txt
 */
// No direct access
defined('_JEXEC') or die;

use \Joomla\CMS\HTML\HTMLHelper;
use \Joomla\CMS\Factory;
use \Joomla\CMS\Uri\Uri;
use \Joomla\CMS\Router\Route;
use \Joomla\CMS\Language\Text;
use \Joomla\CMS\Layout\LayoutHelper;
use \GlennArkell\Component\Gavoting\Administrator\Helper\GavotingHelper;

// load any assets required
$this->document->getWebAssetManager()
    ->usePreset('com_gavoting.gavotingpreset');

// Load admin language file
$lang = Factory::getApplication()->getLanguage();
$lang->load('com_gavoting', JPATH_ADMINISTRATOR);

$user    = GavotingHelper::getSpecificUser();
$canCheckin = $user->authorise('core.manage', 'com_gavoting');
$canNominate  = $user->authorise('core.nominate', 'com_gavoting');
$canEdit = GavotingHelper::canUserEdit($this->item, $user);
$close_noms = GavotingHelper::nominationsClosed();
$today = GavotingHelper::getTodaysDate();
$canNomAny = $this->params->get('allow_anynom', 0);
$reqSecNom = $this->params->get('req_sec_nom', 0);
$allowSecNom = $this->params->get('allow_sec_nom', 0);
$showNominfo = $this->params->get('show_nominfo', 0);

$custFldsEnabled = $this->params->get('custom_fields_enable', 0);
$custFlds = $showNominfo && $custFldsEnabled ? GavotingHelper::getCustomFields('com_gavoting.nomination') : 0;
$this->item->custFlds = $custFldsEnabled && $custFlds ? $custFlds : 0;

if ($canNomAny) {
    // nothing to change here
} elseif (!$canCheckin) {
    $this->form->setFieldAttribute('nomination', 'readonly', 'readonly');
    $this->form->setFieldAttribute('nomination', 'default', $user->id);
}
if (!$canCheckin) {
    $this->form->setFieldAttribute('nom_id', 'readonly', 'readonly');
    $this->form->setFieldAttribute('nom_id', 'default', $user->id);
}
if ($reqSecNom && $allowSecNom) {
    // nothing to change here
} elseif (!$canCheckin) {
    $this->form->setFieldAttribute('sec_id', 'type', 'hidden');
    $this->form->setFieldAttribute('sec_id', 'default', 0);
}
// set fields to be hidden
$this->form->setFieldAttribute('id', 'type', 'hidden');
$this->form->setFieldAttribute('ordering', 'type', 'hidden');
$this->form->setFieldAttribute('state', 'type', 'hidden');
$this->form->setFieldAttribute('checked_out', 'type', 'hidden');
$this->form->setFieldAttribute('checked_out_time', 'type', 'hidden');
$this->form->setFieldAttribute('created_by', 'type', 'hidden');
$this->form->setFieldAttribute('modified_by', 'type', 'hidden');
$this->form->setFieldAttribute('created_date', 'type', 'hidden');
$this->form->setFieldAttribute('modified_date', 'type', 'hidden');
$this->form->setFieldAttribute('nom_date', 'type', 'hidden');
$this->form->setFieldAttribute('agreed', 'type', 'hidden');
$this->form->setFieldAttribute('agreed_date', 'type', 'hidden');
$this->form->setFieldAttribute('votes', 'type', 'hidden');

if ($this->item) {
    $nom_date = !is_null($this->item->nom_date) ? $this->item->nom_date : $today;
    $this->form->setFieldAttribute('nom_date', 'default', $nom_date);
    //$this->form->setValue('nom_date', $nom_date);
    $this->form->setFieldAttribute('state', 'default', $this->item->state);
    $this->form->setFieldAttribute('id', 'default', $this->item->id);
} else {
    $this->form->setFieldAttribute('nom_date', 'default', $today);
    $this->form->setFieldAttribute('id', 'default', 0);
    $this->form->setFieldAttribute('state', 'default', 1);
}
if (!$this->params->get('single_mship',0)) {
    $this->form->setFieldAttribute('oth_mbr', 'type', 'hidden');
    $this->form->setFieldAttribute('spacery', 'type', 'hidden');
    $this->form->setFieldAttribute('spacery', 'class', 'control-group-tight');
    $this->form->setFieldAttribute('spacern', 'type', 'hidden');
    $this->form->setFieldAttribute('spacern', 'class', 'control-group-tight');
}

/*
echo '<pre>Test<br />';
print_r($this->item);
echo '</pre>';
*/
?>

<div class="nomination-edit front-end-edit">
	<?php if ($close_noms) : ?>
		<h3>
			<?php echo Text::_('COM_GAVOTING_NOMINATIONS_CLOSED'); ?>
		</h3>
	<?php else : ?>
		<?php if (!$canNominate) : ?>
			<h3>
				<?php throw new \Exception(Text::_('COM_GAVOTING_ERROR_MESSAGE_NOT_AUTHORISED'), 403); ?>
			</h3>
		<?php else : ?>
			<h2><?php echo Text::_('COM_GAVOTING_TITLE_NOMINATION'); ?></h2>
			<?php if ($reqSecNom) : ?>
                <h3><?php echo Text::_('COM_GAVOTING_NOMINATION_SUBMIT_TEXT'); ?></h3>
	        <?php endif; ?>

			<form id="form-nomination"
				  action="<?php echo Route::_('index.php?option=com_gavoting&task=nominationform.save'); ?>"
				  method="post" class="form-validate form-horizontal" enctype="multipart/form-data">

				<?php echo HTMLHelper::_('bootstrap.startTabSet', 'myTab', array('active' => 'nomination')); ?>
				<?php echo HTMLHelper::_('bootstrap.addTab', 'myTab', 'nomination', Text::_('COM_GAVOTING_TAB_NOMINATION', true)); ?>
				<div class="row-fluid">
					<div class="span10 form-horizontal">
						<fieldset class="adminform">
							<legend><?php echo Text::_('COM_GAVOTING_FIELDSET_NOMINATION'); ?></legend>
							<?php echo $this->form->renderFieldset('general'); ?>
						</fieldset>
					</div>
				</div>
				<?php echo HTMLHelper::_('bootstrap.endTab'); ?>

				<?php if ($custFldsEnabled && $custFlds): ?>
                    <?php echo HTMLHelper::_('bootstrap.addTab', 'myTab', 'custflds', Text::_('COM_GAVOTING_MENUS_MANAGE_NOMINATION', true)); ?>
    				<div class="row-fluid">
    					<div class="span10 form-horizontal">
    						<fieldset class="adminform">
                                <?php echo LayoutHelper::render('default_nominfo', array('data' => $this->item), dirname(__FILE__)); ?>
    						</fieldset>
    					</div>
    				</div>
    				<?php echo HTMLHelper::_('bootstrap.endTab'); ?>
                <?php endif; ?>

				<?php echo HTMLHelper::_('bootstrap.endTabSet'); ?>
                <?php echo $this->form->renderFieldset('sysinfo'); ?>
				<div class="control-group">
					<div class="controls">
						<?php if ($canNominate): ?>
							<button type="submit" class="validate btn btn-primary"><?php echo Text::_('JSUBMIT'); ?></button>
						<?php endif; ?>
						<a class="btn btn-secondary"
						   href="<?php echo Route::_('index.php?option=com_gavoting&task=nominationform.cancel'); ?>"
						   title="<?php echo Text::_('JCANCEL'); ?>"> <?php echo Text::_('JCANCEL'); ?>
						</a>
					</div>
				</div>
	
				<input type="hidden" name="option" value="com_gavoting"/>
				<input type="hidden" name="task" value="nominationform.save"/>
				<?php echo HTMLHelper::_('form.token'); ?>
			</form>
		<?php endif; ?>
	<?php endif; ?>
</div>
