<?php
/**
 * @version    5.1.0
 * @package    Com_Gavoting
 * @author     Glenn Arkell <glenn@glennarkell.com.au>
 * @copyright  2020 Glenn Arkell
 * @license    GNU General Public License version 2 or later; see LICENSE.txt
 */
// No direct access
defined('_JEXEC') or die;

use \Joomla\CMS\HTML\HTMLHelper;
use \Joomla\CMS\Factory;
use \Joomla\CMS\Uri\Uri;
use \Joomla\CMS\Router\Route;
use \Joomla\CMS\Language\Text;
use \Joomla\CMS\Layout\LayoutHelper;
use \Joomla\CMS\User\UserHelper;
use \Joomla\CMS\Date\Date;
use \GlennArkell\Component\Gavoting\Administrator\Helper\GavotingHelper;

// load any assets required
$this->document->getWebAssetManager()
    ->usePreset('com_gavoting.gavotingpreset');

// Load admin language file
$lang = Factory::getApplication()->getLanguage();
$lang->load('com_gavoting', JPATH_ADMINISTRATOR);

$user       = GavotingHelper::getSpecificUser();
$userId     = $user->id;
$listOrder  = $this->state->get('list.ordering', 'nom.name');
$listDirn   = $this->state->get('list.direction', 'ASC');
$canCreate  = $user->authorise('core.create', 'com_gavoting');
$canEdit    = $user->authorise('core.edit', 'com_gavoting');
$canCheckin = $user->authorise('core.manage', 'com_gavoting');
$canScrutin  = $user->authorise('core.scrutin', 'com_gavoting');
$canChair  = $user->authorise('core.chair', 'com_gavoting');
$canChange  = $user->authorise('core.edit.state', 'com_gavoting');
$canDelete  = $user->authorise('core.delete', 'com_gavoting');
$canNominate  = $user->authorise('core.nominate', 'com_gavoting');
$canVote  = $user->authorise('core.vote', 'com_gavoting');
$showVotes = $this->params->get('show_votes',0);
$bio_field = $this->params->get('bio_field','aboutme');
$multiVotes = $this->params->get('multi_votes',0);
$votesField = $this->params->get('votes_field',0);
$votesUser = GavotingHelper::getCustomField($user->id, $votesField);
$votesUser = empty($votesUser) ? 1 : $votesUser;
$limitOffice = $this->params->get('limited_office',0);
$electPeriod = $this->params->get('election_period',0);
$electNumber = $this->params->get('election_number',0);
if($electNumber > 1) { $electPeriod = $electPeriod.'S'; }

// get restrictions
$restrict_nomees = $this->params->get('restrict_nomees',0);
$nomees_user_gp = $this->params->get('nomees_user_gp',0);

$restrict_nomtors = $this->params->get('restrict_nomtors',0);
$nomtors_user_gp = $this->params->get('nomtors_user_gp',0);
if ($canNominate && $restrict_nomtors) {
	$canNominate = GavotingHelper::checkAccessGroup($user->id, $nomtors_user_gp);
}

$restrict_voters = $this->params->get('restrict_voters',0);
$voters_user_gp = $this->params->get('voters_user_gp',0);
if ($canVote && $restrict_voters) {
	$canVote = GavotingHelper::checkAccessGroup($user->id, $voters_user_gp);
}

$req_sec_nom = $this->params->get('req_sec_nom',1);

if ($this->params->get('multi_users', 0)) {
	$linked_field = $this->params->get('linkcust_field', 0);
	$linkedUser = GavotingHelper::getCustomField($user->id, $linked_field);
} else {
    $linkedUser = 0;
}
$hasVoted = GavotingHelper::hasVoted($user->id, 0, $linkedUser);

$showNomInfo = $this->params->get('show_nominfo',0);

$nomsClosed = GavotingHelper::nominationsClosed();
$votingClosed = GavotingHelper::votingClosed();
$votingOpen = GavotingHelper::votingOpen();
$dateFormat = $this->params->get('dateFormat','COM_GAVOTING_DISPLAY_DATETIME');
if (empty($this->params->get('agm_date'))) {
    // don't do dates
} else {
    $ndate = new Date($this->params->get('close_noms'));
    $odate = new Date($this->params->get('open_votes'));
    $cdate = new Date($this->params->get('close_votes'));
    $adate = new Date($this->params->get('agm_date'));
}
$nowdate = new Date();
$tiePositions = GavotingHelper::getTiedNominationPositions();
Factory::getApplication()->setUserState('com_gavoting.tiepositions.data', $tiePositions);

/*
echo '<pre>Test<br />';
print_r(Factory::getApplication()->getUserState('com_gavoting.test.data'));
echo '</pre>';
print_r($this->items);
echo '<br />';
print_r(Factory::getApplication()->getUserState('com_gavoting.raw.nomination.data'));
print_r($fldArray);
echo '<br />';
print_r($fldValue);
*/
?>

<?php if (empty($this->params->get('agm_date'))) : ?>
    <h2><?php echo Text::_('COM_GAVOTING_PARAM_DATES_UNSET'); ?></h2>
<?php else : ?>
    <h2><?php echo Text::_('COM_GAVOTING_TITLE_NOMINATIONS'); ?></h2>
    <p><?php echo Text::_('COM_GAVOTING_IMPORTANT_DATES'); ?> <?php echo HTMLHelper::date($adate, Text::_($dateFormat)); ?>:</p>
    <ul>
    	<li><?php echo Text::_('COM_GAVOTING_CLOSE_NOMS_LABEL').' '.HTMLHelper::date($ndate, Text::_($dateFormat)); ?></li>
    	<li><?php echo Text::_('COM_GAVOTING_OPEN_VOTES_LABEL').' '.HTMLHelper::date($odate, Text::_($dateFormat)); ?></li>
    	<li><?php echo Text::_('COM_GAVOTING_CLOSE_VOTES_LABEL').' '.HTMLHelper::date($cdate, Text::_($dateFormat)); ?></li>
    	<li><?php echo Text::_('COM_GAVOTING_CURRENTDATE_LABEL').' '.HTMLHelper::date($nowdate, Text::_($dateFormat)); ?></li>
    </ul>
    <p><?php echo Text::_('COM_GAVOTING_NOMINATION_EXPLANATION'); ?></p>
    
    <?php if($limitOffice) : ?>
        <p><?php echo Text::sprintf('COM_GAVOTING_LIMITED_OFFICE_TERM', Text::_($electNumber), Text::_($electPeriod)); ?></p>
    <?php endif; ?>
    <form action="<?php echo htmlspecialchars(Uri::getInstance()->toString()); ?>" method="post"
          name="adminForm" id="adminForm" class="com-content-category__articles">
    
    	<?php echo LayoutHelper::render('default_filter', array('view' => $this), dirname(__FILE__)); ?>
    
        <div class="com-contact-categories categories-list">
    	<table class="table table-striped" id="nominationList">
    		<thead>
    		<tr>
    			<th width="25%" class=''>
    				<?php echo HTMLHelper::_('grid.sort',  'COM_GAVOTING_NOMINATIONS_POSITION_ID', 'pos.pos_name', $listDirn, $listOrder); ?>
    			</th>
    			<th width="25%" class='center'>
    				<?php echo HTMLHelper::_('grid.sort',  'COM_GAVOTING_NOMINATIONS_NOMINATION', 'a.nom_name', $listDirn, $listOrder); ?>
    			</th>
    			<th width="15%" class="center">
    				<?php echo HTMLHelper::_('grid.sort',  'COM_GAVOTING_NOMINATIONS_NOM_ID', 'nom.name', $listDirn, $listOrder); ?>
    			</th>
    			<?php if ($req_sec_nom): ?>
    				<th width="15%">
    					<?php echo HTMLHelper::_('grid.sort',  'COM_GAVOTING_NOMINATIONS_SEC_ID', 's.name', $listDirn, $listOrder); ?>
    				</th>
    				<?php if ($canCheckin || $showVotes): ?>
    					<th width="15%" class="center">
    						<?php echo HTMLHelper::_('grid.sort',  'COM_GAVOTING_NOMINATIONS_VOTES', 'a.votes', $listDirn, $listOrder); ?>
    					</th>
    				<?php endif; ?>
    
    				<?php if ($canCheckin || $canDelete): ?>
    					<th width="15%" class="center">
    					<?php echo Text::_('COM_GAVOTING_ACTIONS'); ?>
    					</th>
    				<?php endif; ?>
    			<?php else : ?>
    				<?php if ($canCheckin || $showVotes): ?>
    					<th width="15%" class="center">
    						<?php echo HTMLHelper::_('grid.sort',  'COM_GAVOTING_NOMINATIONS_VOTES', 'a.votes', $listDirn, $listOrder); ?>
    					</th>
    				<?php endif; ?>
    
    				<?php if ($canCheckin || $canDelete): ?>
    					<th width="30%" class="center">
    					<?php echo Text::_('COM_GAVOTING_ACTIONS'); ?>
    					</th>
    				<?php endif; ?>
    			<?php endif; ?>
    
    		</tr>
    		</thead>
    		<tfoot>
    		<tr>
    			<td colspan="<?php echo isset($this->items[0]) ? count(get_object_vars($this->items[0])) : 10; ?>">
    				<?php echo $this->pagination->getListFooter(); ?>
    			</td>
    		</tr>
    		</tfoot>
    		<tbody>
    		<?php foreach ($this->items as $i => $item) : ?>
    			<?php 
    				$canEdit = $user->authorise('core.edit', 'com_gavoting');
    				if (!$canEdit && $user->authorise('core.edit.own', 'com_gavoting')) {
    					$canEdit = GavotingHelper::getSpecificUser()->id == $item->nomination;
    				}
    
    	            if (!$item->agreed) { $nomStyle = ' style="color:red;"'; } else { $nomStyle = ''; }
    				$showProf = false;
    				//$prof = $item->nom_name.' joined on '.$item->joinedDate.'.&#013;&#010;';
    				//$prof .= '======================================== &#013;&#010;&#013;&#010;';
    				$prof = Text::sprintf('COM_GAVOTING_BIO_HEADER',$item->nom_name,$item->joinedDate);
    				$prof .= '&#013;&#010; ======================================== &#013;&#010;&#013;&#010;';
    				$profile = GavotingHelper::getProfileValue($item->nomination, $bio_field);
    				if (isset($profile)) {
    					$prof .= str_replace('\r\n', '&#013;&#010;',trim($profile->profile_value,'"'));
    					$showProf = true;
    				}
    
    				$nomName = $item->nom_name;
                    // get nomination info if set to show
                    if ($showNomInfo) {
                        // just leave as name if no custom fields set
    
                    }
    
    			?>
    
    			<tr class="row<?php echo $i % 2; ?>">
    
    				<td<?php echo $nomStyle; ?>>
    					<?php if (isset($item->checked_out) && $item->checked_out) : ?>
    						<?php echo HTMLHelper::_('jgrid.checkedout', $i, $item->uEditor, $item->checked_out_time, 'nominations.', $canCheckin); ?>
    					<?php endif; ?>
    					<?php if ($canEdit): ?>
    						<a href="<?php echo Route::_('index.php?option=com_gavoting&view=nomination&id='.(int) $item->id); ?>">
    							<?php echo $this->escape($item->position_id_name); ?>
    						</a>
    					<?php else : ?>
    						<?php echo $this->escape($item->position_id_name); ?>
    					<?php endif; ?>
    				</td>
    				<td class="left"<?php echo $nomStyle; ?>>
    
    					<?php if ($showProf): ?>
    						<?php echo '<i class="icon-info" data-html="true" title="'.$prof.'"></i> &nbsp; '.$nomName; ?>
    					<?php else : ?>
    						<?php echo $nomName; ?>
    					<?php endif; ?>
    
    					<?php if ($item->agreed == 0 && $item->nomination == $userId && !$votingOpen): ?>
    						<a href="<?php echo Route::_('index.php?option=com_gavoting&task=nomination.agree&id='.(int) $item->id, false, 2); ?>"
    							class="btn btn-mini" type="button" title="<?php echo Text::_('COM_GAVOTING_ACCEPT_NOMINATION'); ?>">
    							<i class="icon-publish" ></i>
    						</a>
    						<a href="<?php echo Route::_('index.php?option=com_gavoting&task=nomination.reject&id='.(int) $item->id, false, 2); ?>"
    							class="btn btn-mini" type="button" title="<?php echo Text::_('COM_GAVOTING_REJECT_NOMINATION'); ?>">
    							<i class="icon-unpublish" ></i>
    						</a>
    					<?php endif; ?>
    				</td>
    				<td class="left"<?php echo $nomStyle; ?>>
    					<?php echo $item->nominator_name; ?>
    				</td>
    				<?php if ($req_sec_nom): ?>
    					<td class="left"<?php echo $nomStyle; ?>>
    						<?php if ($item->sec_id == 0): ?>
    							<?php if ($item->nom_id != $userId && $item->nomination != $userId): ?>
    								<a href="<?php echo Route::_('index.php?option=com_gavoting&task=nomination.seconder&id='.(int) $item->id.'&user_id='.(int) $userId, false, 2); ?>" 
    									class="btn btn-mini" type="button" title="<?php echo Text::_('COM_GAVOTING_AGREE_TO_SECOND'); ?>">
    									<i class="icon-publish" ></i>
    								</a>
    							<?php endif; ?>
    						<?php else : ?>
    							<?php echo $item->seconder_name; ?>
    						<?php endif; ?>
    					</td>
    				<?php endif; ?>
    				<?php if ($canCheckin || $showVotes): ?>
    					<td class="center">
    						<?php echo $item->votes; ?>
    					</td>
    				<?php endif; ?>
    
    				<?php if ($canCheckin || $canDelete): ?>
    					<td class="center">
    						<?php if ($canCheckin): ?>
    							<a href="<?php echo Route::_('index.php?option=com_gavoting&task=nominationform.edit&id=' . $item->id, false, 2); ?>" class="btn-secondary btn-mini" type="button" title="Edit Record"><i class="icon-edit" ></i></a> &nbsp; &nbsp;
    						<?php endif; ?>
    						<?php if ($canDelete): ?>
    							<a href="<?php echo Route::_('index.php?option=com_gavoting&task=nominationform.remove&id=' . $item->id, false, 2); ?>" class="btn-secondary btn-mini delete-button" type="button" title="Delete Record"><i class="icon-trash" ></i></a> &nbsp; &nbsp;
    							<a href="<?php echo Route::_('index.php?option=com_gavoting&task=nominationform.archive&id=' . $item->id, false, 2); ?>" class="btn-secondary btn-mini" type="button" title="Archive Record"><i class="icon-archive" ></i></a>
    						<?php endif; ?>
    					</td>
    				<?php endif; ?>
    
    			</tr>
    		<?php endforeach; ?>
    		</tbody>
    	</table>
    
        <p class="small"><?php echo Text::_('COM_GAVOTING_NOMS_NOT_AGREED_YET'); ?></p>
    
        </div>
    	<?php if ($canNominate && !$nomsClosed) : ?>
    		<a href="<?php echo Route::_('index.php?option=com_gavoting&task=nominationform.edit&id=0', false, 0); ?>"
    		   class="btn btn-success btn-small"><i class="icon-plus"></i> <?php echo Text::_('COM_GAVOTING_ADD_NOMINATION'); ?>
    		</a>
    	<?php endif; ?>
    	<?php if ($hasVoted) : ?>
    		<p><?php echo Text::_('COM_GAVOTING_ALREADY_VOTED'); ?></p>
    	<?php else : ?>
    		<?php if ($canVote && $votingOpen && !$votingClosed) : ?>
    			<a href="<?php echo Route::_('index.php?option=com_gavoting&task=voterform.edit&id=0', false, 0); ?>"
    			   class="btn btn-warning btn-small"><i class="icon-fas fa-person-booth"></i> <?php echo Text::_('COM_GAVOTING_CAST_VOTE'); ?>
    			</a>
    		<?php endif; ?>
    	<?php endif; ?>
    
    	<?php if (($canScrutin || $canChair) && $votingClosed) : ?>
        	<?php if ($canChair && !empty($tiePositions)) : ?>
        		<a href="<?php echo Route::_('index.php?option=com_gavoting&task=voterform.castDecisionVote', false, 0); ?>"
        		   class="btn btn-incident btn-small"><i class="icon-fas fa-person-booth"></i> <?php echo Text::_('COM_GAVOTING_CAST_DECIDING_VOTE'); ?>
        		</a>
            <?php else : ?>
        		<a href="<?php echo Route::_('index.php?option=com_gavoting&task=voterform.voteDecision', false, 0); ?>"
        		   class="btn btn-danger btn-small"><i class="icon-redo"></i> <?php echo Text::_('COM_GAVOTING_DECLARE_WINNERS'); ?>
        		</a>
            <?php endif; ?>
        <?php endif; ?>
    	<input type="hidden" name="task" value=""/>
    	<input type="hidden" name="boxchecked" value="0"/>
    	<input type="hidden" name="filter_order" value="<?php echo $listOrder; ?>"/>
    	<input type="hidden" name="filter_order_Dir" value="<?php echo $listDirn; ?>"/>
    	<?php echo HTMLHelper::_('form.token'); ?>
    </form>
<?php endif; ?>

<?php if($canDelete) : ?>
<script type="text/javascript">

	jQuery(document).ready(function () {
		jQuery('.delete-button').click(deleteItem);
	});

	function deleteItem() {

		if (!confirm("<?php echo Text::_('COM_GAVOTING_DELETE_MESSAGE'); ?>")) {
			return false;
		}
	}
</script>
<?php endif; ?>
