<?php
/**
 * @version    5.1.0
 * @package    Com_Gavoting
 * @author     Glenn Arkell <glenn@glennarkell.com.au>
 * @copyright  2020 Glenn Arkell
 * @license    GNU General Public License version 2 or later; see LICENSE.txt
 */
// No direct access
defined('_JEXEC') or die;

use \Joomla\CMS\HTML\HTMLHelper;
use \Joomla\CMS\Factory;
use \Joomla\CMS\Uri\Uri;
use \Joomla\CMS\Router\Route;
use \Joomla\CMS\Language\Text;
use \Joomla\CMS\Layout\LayoutHelper;
use \GlennArkell\Component\Gavoting\Administrator\Helper\GavotingHelper;

// load any assets required
$this->document->getWebAssetManager()
    ->usePreset('com_gavoting.gavotingpreset');

// Load admin language file
$lang = Factory::getApplication()->getLanguage();
$lang->load('com_gavoting', JPATH_ADMINISTRATOR);

$user       = GavotingHelper::getSpecificUser();
$userId     = $user->get('id');
$listOrder  = $this->state->get('list.ordering', 'a.id');
$listDirn   = $this->state->get('list.direction', 'ASC');
$canCreate  = $user->authorise('core.create', 'com_gavoting');
$canEdit    = $user->authorise('core.edit', 'com_gavoting');
$canCheckin = $user->authorise('core.manage', 'com_gavoting');
$canChange  = $user->authorise('core.edit.state', 'com_gavoting');
$canDelete  = $user->authorise('core.delete', 'com_gavoting');

/*
echo '<pre>Test<br />';
print_r(Factory::getApplication()->getUserState('com_gavoting.test.files'));
echo '</pre>';
print_r(Factory::getApplication()->getUserState('com_gavoting.test.data'));
*/
?>

<h2><?php echo Text::_('COM_GAVOTING_TITLE_POSITION'); ?></h2>

<form action="<?php echo htmlspecialchars(Uri::getInstance()->toString()); ?>" method="post"
      name="adminForm" id="adminForm" class="com-content-category__articles">

	<?php
        // display standard joomla filter files
        //echo LayoutHelper::render('joomla.searchtools.default', array('view' => $this));
        // display component view specific filter files
        echo LayoutHelper::render('default_filter', array('view' => $this), dirname(__FILE__));
    ?>

    <div class="com-contact-categories categories-list">
	<table class="table table-striped" id="positionList">
		<thead>
		<tr>
			<th class=''>
				<?php echo HTMLHelper::_('grid.sort',  'COM_GAVOTING_POSITIONS_POS_NAME', 'a.pos_name', $listDirn, $listOrder); ?>
			</th>
			<th  class="center">
				<?php echo HTMLHelper::_('grid.sort',  'COM_GAVOTING_POSITIONS_POS_QTY', 'a.pos_qty', $listDirn, $listOrder); ?>
			</th>
			<th class='center'>
				<?php echo HTMLHelper::_('grid.sort',  'COM_GAVOTING_POSITIONS_CAT_ID', 'a.cat_id_name', $listDirn, $listOrder); ?>
			</th>
			<th class='center'>
				<?php echo HTMLHelper::_('grid.sort',  'COM_GAVOTING_POSITIONS_ELECT_DATE', 'a.elect_date', $listDirn, $listOrder); ?>
			</th>
			<th class='center'>
				<?php echo HTMLHelper::_('grid.sort',  'COM_GAVOTING_POSITIONS_ELECTED', 'a.elected', $listDirn, $listOrder); ?>
			</th>

			<?php if ($canCheckin): ?>
				<th width="15%" class="center"><?php echo Text::_('COM_GAVOTING_ACTIONS'); ?></th>
			<?php endif; ?>

		</tr>
		</thead>
		<tfoot>
		<tr>
			<td colspan="<?php echo isset($this->items[0]) ? count(get_object_vars($this->items[0])) : 10; ?>">
				<?php echo $this->pagination->getListFooter(); ?>
			</td>
		</tr>
		</tfoot>
		<tbody>
		<?php foreach ($this->items as $i => $item) : ?>
			<?php 
                $canEdit = GavotingHelper::canUserEdit($item);
                if ($item->jd_id) {
                    $jd = GavotingHelper::getArticle($item->jd_id);
                } else {
                    $jd = 0;
                }
            ?>

			<tr class="row<?php echo $i % 2; ?>">

				<td>
					<?php if (isset($item->checked_out) && $item->checked_out) : ?>
						<?php echo HTMLHelper::_('jgrid.checkedout', $i, $item->uEditor, $item->checked_out_time, 'positions.', $canCheckin); ?>
					<?php endif; ?>
					<a href="<?php echo Route::_('index.php?option=com_gavoting&view=position&id='.(int) $item->id); ?>">
						<?php echo $item->pos_name; ?>
					</a> &nbsp; &nbsp;
					<?php 
                        if ($jd) {
                            $jdLink = GavotingHelper::getHTTPQuery(null, 'view', 'article', 'id', $jd->id);
                            $jdLink = GavotingHelper::getHTTPQuery($jdLink, null, null, 'tmpl', 'component');
                            $jdLink = GavotingHelper::getHTTPQuery($jdLink, null, null, 'layout', 'modal');
                            $modalParams = array(
                                'url'        => 'index.php?'.http_build_query($jdLink, '', '&amp;'),
                                'title'      => Text::_('COM_GAVOTING_FORM_LBL_POSITION_JD_ID'),
                                'closeButton'=> true,
                                'modalWidth' => 60,
                                'bodyHeight' => 45,
                                'backdrop'   => 'static'
                                );
                            $html = '<a href="#modalJD'.$jd->id.'" data-bs-toggle="modal">';
                            $html .= '<i class="icon-info" title="'.Text::_('COM_GAVOTING_FORM_LBL_POSITION_JD_ID').'"></i></a>';
                            echo $html .= HTMLHelper::_('bootstrap.renderModal', 'modalJD'.$jd->id, $modalParams);
    					}
                    ?>
				</td>
				<td class="center">
					<?php echo $item->pos_qty; ?>
				</td>
				<td class="center">
					<?php echo $item->cat_id_name; ?>
				</td>
				<td class="center">
					<?php if (!empty($item->elect_date) && $item->elect_date != '0000-00-00 00:00:00') {echo HtmlHelper::date($item->elect_date, Text::_('COM_GAVOTING_DISPLAY_DATE'));} ?>
				</td>
				<td class="center">
					<?php echo $item->elected; ?>
				</td>

				<?php if ($canCheckin): ?>
					<td class="center">
						<?php if ($canEdit): ?>
							<a href="<?php echo Route::_('index.php?option=com_gavoting&task=positionform.edit&id=' . $item->id, false, 2); ?>" class="btn-secondary btn-mini" type="button" title="Edit Record"><i class="icon-edit" ></i></a> &nbsp; &nbsp;
						<?php endif; ?>
						<?php if ($canDelete): ?>
							<a href="<?php echo Route::_('index.php?option=com_gavoting&task=positionform.remove&id=' . $item->id, false, 2); ?>" class="btn-secondary btn-mini delete-button" type="button" title="Delete Record"><i class="icon-trash" ></i></a> &nbsp; &nbsp;
							<a href="<?php echo Route::_('index.php?option=com_gavoting&task=positionform.archive&id=' . $item->id, false, 2); ?>" class="btn-secondary btn-mini" type="button" title="Archive Record"><i class="icon-archive" ></i></a>
						<?php endif; ?>
					</td>
				<?php endif; ?>

			</tr>
		<?php endforeach; ?>
		</tbody>
	</table>
    </div>
	<?php if ($canCheckin) : ?>
		<a href="<?php echo Route::_('index.php?option=com_gavoting&task=positionform.edit&id=0', false, 0); ?>"
		   class="btn btn-success btn-small"><i class="icon-plus"></i> <?php echo Text::_('COM_GAVOTING_ADD_ITEM'); ?>
		</a>
	<?php endif; ?>

	<input type="hidden" name="task" value=""/>
	<input type="hidden" name="boxchecked" value="0"/>
	<input type="hidden" name="filter_order" value="<?php echo $listOrder; ?>"/>
	<input type="hidden" name="filter_order_Dir" value="<?php echo $listDirn; ?>"/>
	<?php echo HTMLHelper::_('form.token'); ?>
</form>

<?php if($canDelete) : ?>
<script type="text/javascript">

	jQuery(document).ready(function () {
		jQuery('.delete-button').click(deleteItem);
	});

	function deleteItem() {
		if (!confirm("<?php echo Text::_('COM_GAVOTING_DELETE_MESSAGE'); ?>")) {
			return false;
		}
	}
</script>
<?php endif; ?>
