<?php
/**
 * @version    5.1.0
 * @package    Com_Gavoting
 * @author     Glenn Arkell <glenn@glennarkell.com.au>
 * @copyright  2020 Glenn Arkell
 * @license    GNU General Public License version 2 or later; see LICENSE.txt
 */
// No direct access
defined('_JEXEC') or die;

use \Joomla\CMS\HTML\HTMLHelper;
use \Joomla\CMS\Factory;
use \Joomla\CMS\Uri\Uri;
use \Joomla\CMS\Router\Route;
use \Joomla\CMS\Language\Text;
use \Joomla\CMS\Date\Date;
use \Joomla\CMS\Layout\LayoutHelper;
use \GlennArkell\Component\Gavoting\Administrator\Helper\GavotingHelper;

// load any assets required
$this->document->getWebAssetManager()
    ->usePreset('com_gavoting.gavotingpreset')
    ->useScript('com_gavoting.refreshsinglejs');
//    ->useScript('com_gavoting.refreshmultijs', $this->item->nominations);

// Load admin language file
$lang = Factory::getApplication()->getLanguage();
$lang->load('com_gavoting', JPATH_ADMINISTRATOR);

$tiePositions = Factory::getApplication()->getUserState('com_gavoting.tiepositions.data');
Factory::getApplication()->setUserState('com_gavoting.tiepositions.data', null);
$this->item->deciderVote = false;

$user    = GavotingHelper::getSpecificUser();
$canVote  = $user->authorise('core.vote', 'com_gavoting');
$canChair  = $user->authorise('core.chair', 'com_gavoting');
$this->item->canChair = $canChair;
$this->item->voting_user = $user->id;
$this->item->canEdit = GavotingHelper::canUserEdit($this->item, $user);

$allow_proxy = $this->params->get('allow_proxy', 0);
$restrict_voters = $this->params->get('restrict_voters',0);
$voters_user_gp = $this->params->get('voters_user_gp',0);
if ($canVote && $restrict_voters) {
	$canVote = GavotingHelper::checkAccessGroup($user->id, $voters_user_gp);
}

$pref_voting = $this->params->get('pref_voting',0);

$multiVotes = $this->params->get('multi_votes',0);
$votesField = $this->params->get('votes_field',0);
$votesUser = GavotingHelper::getCustomField($user->id, $votesField);
$votesUser = empty($votesUser) ? 1 : $votesUser;

//$today = GavotingHelper::getTodaysDate();
$this->item->nomsClosed = GavotingHelper::nominationsClosed();
$this->item->votingClosed = GavotingHelper::votingClosed();
$this->item->votingOpen = GavotingHelper::votingOpen();

if ($this->params->get('multi_users', 0)) {
	$linked_field = $this->params->get('linkcust_field', 0);
	$linkedUser = GavotingHelper::getCustomField($user->id, $linked_field);
} else {
    $linkedUser = 0;
}

$this->item->hasVoted = GavotingHelper::hasVoted($user->id, 0, $linkedUser);
$this->item->params = $this->params;
$this->item->canSave = $this->canSave;
$this->item->form = $this->form;

/* -------   Check if principal/owner of vote has delegated a proxy  ------- */
/* -------   returns an object of the proxy record or boolean on null  ----- */
$this->item->proxyVote = GavotingHelper::checkProxySet($user->id);
$proxySet = (isset($this->item->proxyVote) && !empty($this->item->proxyVote)) ? true : false;

$prevPos='';
$cntr=0;

if (!empty($tiePositions) && $this->item->votingClosed && $canChair) {
    // change settings so a decider vote can be cast
    $this->item->votingClosed = false;
    $this->item->deciderVote = true;
}

/*
echo '<pre>Test<br />';
print_r($this->item->posIDs);
echo '</pre>';
print_r(Factory::getApplication()->getUserState('com_gavoting.test.data'));
*/
?>

<div class="voter-edit front-end-edit">
<?php if ($this->item->hasVoted && !$canChair): ?>
	<h2><?php echo Text::_('COM_GAVOTING_NOMINATIONS_ALREADY_VOTED'); ?></h2>
<?php elseif (!$canVote): ?>
	<h2><?php echo Text::_('COM_GAVOTING_ERROR_MESSAGE_NOT_AUTHORISED'); ?></h2>
<?php else: ?>

	<?php if ($allow_proxy && $proxySet): ?>
		<h2><?php echo Text::sprintf('COM_GAVOTING_PROXY_ALREADY_SET', $this->item->proxyVote->name); ?></h2>
	<?php else: ?>

		<?php if (!$this->item->nomsClosed): ?>
			<h2><?php echo Text::_('COM_GAVOTING_NOMINATIONS_STILL_OPEN'); ?></h2>
		<?php else: ?>
			<?php if (!$this->item->canEdit) : ?>
				<h3>
					<?php throw new \Exception(Text::_('COM_GAVOTING_ERROR_MESSAGE_NOT_AUTHORISED'), 403); ?>
				</h3>
			<?php else : ?>
				<?php if ($this->item->votingClosed): ?>
					<h2><?php echo Text::_('COM_GAVOTING_VOTING_CLOSED'); ?></h2>
				<?php else: ?>
					<?php if (!$this->item->votingOpen): ?>
						<h2><?php echo Text::_('COM_GAVOTING_VOTING_NOT_OPEN'); ?></h2>
					<?php else : ?>
						<?php if (!empty($this->item->nominations) && $this->item->nominations > ''): ?>
		
                            <?php
                                if ($multiVotes) {
                                    echo '<h3 class="center">'.Text::sprintf('COM_GAVOTING_MULTIPLE_VOTES', $votesUser).'</h3>';
                                }
                            ?>

							<?php /* **************************   Voting Form  *********************************/  ?>
							<?php if ($pref_voting): ?>
    							<?php echo LayoutHelper::render('default_prefer', array('data' => $this->item), dirname(__FILE__)); ?>
							<?php else : ?>
    							<?php echo LayoutHelper::render('default_voting', array('data' => $this->item), dirname(__FILE__)); ?>
		                    <?php endif; ?>
						<?php else : ?>
							<?php echo Text::_('COM_GAVOTING_NO_NOMINATIONS'); ?>
						<?php endif; ?>
					<?php endif; ?>
				<?php endif; ?>
			<?php endif; ?>
		<?php endif; ?>
	<?php endif; ?>
<?php endif; ?>
</div>
